% Simple refletion by mirror
% 
% src -> mirror -> detector
% 
%{ 
% example 
% build optical system
mir = MirrorRef;
% print information
mir.printAll
% get field at the detector with default window size
% [E, xy] = mir.getField('field-name');
% E(yj,xi) = field at (xy(xi), xy(yj))
[E, x] = mir.getField( 'reflMon-1-i');
figure(1)
pcolor( x, x, log(abs(E).^2) ); shading flat; axis equal; colorbar;
% get field at the detector with larger window size, 0.2m visible window
[E, x] = mir.getField( 'reflMon-1-i', 0.1 );
figure(2)
pcolor( x, x, log(abs(E).^2) ); shading flat; axis equal; colorbar;
% get field at different locations (virtually moving the target)
[E0, x0] = mir.getField( 'MC2-HRy-o' ); % field on MC2 going toward detector
[E60, x60] = mir.getField( 'reflMon-1-i', 0.1, 60 ); % window size 20cm, at 60m from source
mir.gaussFit( 'reflMon-1-i' )
figure(3)
semilogy( x0.^2, abs(E0(end/2,:)).^2, 'b', x60.^2, abs(E60(end/2,:)).^2, 'r', ...
x.^2, abs(E(end/2,:)).^2, 'g', ...
x.^2, abs(E(end/2,end/2)^2)*exp(-2*x.^2/0.0051^2), 'cs' );
legend('0m', '60m', '137m', 'gauss fit');
axis([0 1e-3 1e-6 1e5])
%}

%% object defining the setup
classdef MirrorRef < FFTIFO
    
    methods
        %% to support runtime variables
        function obj = MirrorRef( varargin )
            obj@FFTIFO( varargin{:} );
        end

        %% definition of IFO
        function defineIFO( obj, varargin )
            
            %% variables, which can be changed interactively by UtilTK.setVars
            incAng = 0;     % incident angle to the mirror
            Ldet = 143;     % distance between the mirror and detector
            RoCMC2 = 185.1; % RoC of the mirror
            res = 0.001;    % resolution of the miror map
            
            %% this is to change variables defined above by using runtime argument, 
            % like mir = MirrorRef( 'incAng', 0.043 / 143.424 );
            eval( UtilTK.setVars( who, true, varargin{:} ) );
            
            %% setup optics
            % add laser. "help FFTIFO.addLaser" will show all parameters
            obj.addLaser( 'src' );
            
            % add a mirror. "help FFTIFO.addLaser" will show all parameters
            %
            % If 'incAngle' (normal incident angle to the mirror, 0 for normal incident) is not specified, 
            % the mirror is like a test mass, i.e., 2 inputs and 2 outputs,
            %   HR                AR
            %  HR-i ==> || | ==> AR-o
            %  HR-o <== || | <== AR-i
            %
            % If 'incAngle' is specified, the mirror is like a BS, i.e., 4 inputs and 4 outputs,
            %            HRy-o  HRy-i
            %                 ^ v
            %                 | |// /
            %                 | // /
            %       HRx-i >--- // / ---> ARx-o
            %       HRx-o <-- // / <---- ARx-i
            %                // / |
            %               // /| |
            %            HR  AR | ^
            %                   v ARy-i
            %                  ARy-o
            %
            obj.addMirror( 'MC2', 'invRoC', 1/RoCMC2, 'Thickness', 0.0, 'T', 0, 'incAngle', incAng, 'resolution', res );
            
            % add detector
            obj.addDetector( 'reflMon' );
            
            %% connect optics   : help FFTIFO.addProp
            % addProp(A,B) calls addOneProp(A,B,Lab) and addOneProp(B,A,Lab) if
            % propagation from B to A makes sense
            obj.addOneProp( '', 'src-out', 'MC2-HRx' );
            obj.addProp( '', 'MC2-HRy', 'reflMon-1', Ldet );  % detector name is somehow 'name-1', bad legacy to be fixed
            
            %% define the HR surface  : help FFTIFO.setHRfiles, FFTIFT.setTRfiles, FFTIFO.setARfiles
            obj.setHRfiles( 'MC2', 'MC_end_mirror_map_surface_2.col', '', 0.130/2, 185.1 );

        end
        
        %% define the input beam mode
        % fillQvals( fieldID, 1/qVal ) specified the mode. Then this mode is
        % propagated to all fields and this mode base is used for mode
        % calculations, like HGFrac.
        % This setting is necessary because there is no cavity which can be used to
        % define a default mode. For an optical system which has a
        % reasonable cavity which can be used to define modes, that is used
        % as the default mode.
        function setupBaseMode( obj, varargin )
            RoCMC2 = 185.1;
            w = 0.0108;
            eval( UtilTK.setVars( who, true, varargin{:} ) );
            
            % invq : 1/R - i lambda/pi 1/w^2 => -1/R - i lambda/pi 1/w^2
            qInv = 1/RoCMC2 - 1i*1.064e-6/pi/w^2;
            % field ID of laser source
            root = obj.optics.findLaser;
            % mode is defined by specifying the field ID and the q value
            obj.cavities.fillQvals( root, qInv );
        end
    end
end