%% Simple FP cavity simulation
%
% how to define a new IFO
%
% 1) copy the following template code into a file IFOname.m, 
%    whete IFOname can be anything, like FPIFO.m in this case.
% 2) change IFOname to whatever you used for the file name, like FPIFO
% 3) Define your IFO configuration
%    a) Use addLaser, addMirror and addDetector to define optics
%    b) Use addProp to define the connection among optics
%    c) Use setHRfiles to define the details of the HR surface
%
%
% template code
% % ===> from here
% classdef IFOname < FFTIFO
%     methods
%         function obj = IFOname( varargin )
%             obj@FFTIFO( varargin{:} );
%         end
% 
%         function defineIFO( obj, varargin )
%             % set parameters
%             eval( UtilTK.setVars( who, true, varargin{:} ) );
%             % add optics
%             % connection optics
%             % setup details of optics, like maps
%         end
%     end
% end
% % <=== to here
%
%% example run
%{
% build cavity
% force the FFT window size to be 60cm (internally 120cm) so that power
% power distribution can be calculated up to 30cm
% if you are interested in only powers in mirrors, just fp=FPIFO will do.
fp = FPIFO( 'WfftUser', 0.6 );
% lock the cavity and calculate field
fp.lock;
% calculate fields at various locations
help FFTIFO.getField
% field on ITM HR input, i.e., from ETM
[E0, x0] = fp.getField('ITM-HR-i');
% field with larger FFT window to see the rind with 0.6m
[E1, x1] = fp.getField('ITM-HR-i', 1.4 );
% fields at 2000m and 3000m from the source, i.e., ETM
[E2, x2] = fp.getField('ITM-HR-i', 1.4, 2000 );
[E3, x3] = fp.getField('ITM-HR-i', 1.4, 3000 );
figure(1)
subplot(2,2,1)
pcolor( x0, x0, log(abs(E0)) ); shading flat; axis equal; colorbar;
title('4km, small Wfft');
subplot(2,2,2)
pcolor( x1, x1, log(abs(E1)) ); shading flat; axis equal; colorbar;
title('4km, large Wfft');
subplot(2,2,3)
pcolor( x3, x3, log(abs(E3)) ); shading flat; axis equal; colorbar;
title('3km');
subplot(2,2,4)
pcolor( x2, x2, log(abs(E2)) ); shading flat; axis equal; colorbar;
title('2km');

% power distriution
for ir = 1:6
  subpower(1,ir) = 0.05*ir;
  % power in radius r
  subpower(2,ir) = fp.powerInR('ITM-HR-i', subpower(1,ir));
end
figure(2)
semilogy( subpower(1,2:end), subpower(2,2:end)-subpower(2,1:end-1) );

format longe

% round trip loss
fp.roundtripLoss
% field shape on ETM
fp.gaussFit('ETM-HR-o')
% mode in the cavity
fp.HGFrac('ETM-HR-o', 0:4,0:4)
fp.LGFrac('ETM-HR-o', 0:5,0)

% heat ITM/ETM
% Add thermal bump on ITM and ETM, each absorbing 0.5W.
fp.updateMirMaps('ITM', 'PcoatI', 0.5);
fp.updateMirMaps('ETM', 'PcoatE', 0.5);
% second argument 0 means 'suppress progress inforamtion printing'
fp.lock( 1e-6, 0 );

% round trip loss
fp.roundtripLoss
% field shape on ETM
fp.gaussFit('ETM-HR-o')
% mode in the cavity
fp.HGFrac('ETM-HR-o', 0:4,0:4)
fp.LGFrac('ETM-HR-o', 0:5,0)

% RF scan of a FP
% use large ITM transmittance to make finesse low and the resonance broad
fp = FPIFO('TITM', 0.14);
% this does the RF scan
dat= scanRF(fp, 10);
figure(3)
subplot(2,2,1)
semilogy( dat(:,1), dat(:,2) )
title('Power vs RF')
subplot(2,2,2)
semilogy( dat(:,1), dat(:,3))
title('round trip loss');
subplot(2,2,3)
semilogy( dat(:,1), dat(:,4))
title('higher order mode fraction')
subplot(2,2,4)
semilogy( dat(:,1), dat(:,[5:9]))
title('LG20 to LG50 mode fractions')
%}

%% main code
classdef FPIFO < FFTIFO
    
    methods
        %% this definition allows to pass parameters at runtime, like
        %  fp = FPIFO('tiltE', 1e-8);
        function obj = FPIFO( varargin )
            obj@FFTIFO( varargin{:} );
        end

        function defineIFO( obj, varargin )
%% parameters
            % if you want to speficy the FFT window size, use this.
            % The internal FFT window size is double of this.
            WfftUser = 0;
            FPIFOParams;
            
            % parameters can be changed by runtime arguments, 
            % like fp = FPIFO('tiltE', 1e-8)
            eval( UtilTK.setVars( who, true, varargin{:} ) );

%% add optics
            % laser source, can be LG or HG
            obj.addLaser( 'src', 'power', 1 );
            
            % ITM
            obj.addMirror( 'ITM', 'invRoC', 1/ITMRoC, 'Aperture', ATM, 'Thick', dTM, ...
                'T', TITM, 'Lhr', Loss_I_HR, 'Lar', Loss_I_AR, 'WfftUser', WfftUser );
            % ETM
            obj.addMirror( 'ETM', 'invRoC', 1/ETMRoC, 'Aperture', ATM, 'Thick', dTM, ...
                'T', TETM, 'Lhr', Loss_E_HR, 'resolution', resolution );
            
            % detector behind ETM
            obj.addDetector( 'trans' );
            
%% define connections among optics
            obj.addProp( '', 'src', 'ITM-AR' );
            obj.addProp( 'FPprop', 'ITM-HR', 'ETM-HR', Lcav);
            obj.addProp( '', 'ETM-AR', 'trans' );
            
            %% find beam size on ITM and ETM
            [wI, wE] = obj.findBeamSize;
            
%% use the measured maps
            % ITM HR surface
            if ~isempty( ITMmap )
                obj.setHRfiles( 'ITM', ITMmap, 'mapMul * map+y*tiltI', 0.16, ITMRoC, 0, 'tiltI', tiltI, 'mapMul', mapMul  );
            end
            % ETM HR surface
            if ~isempty( ETMmap )
                obj.setHRfiles( 'ETM', ETMmap, 'mapMul * map+y*tiltE', 0.16, ETMRoC, 0, 'tiltE', tiltE, 'mapMul', mapMul );
            end
 
            % thermal deformation
            % setHRfiles can be added multiple times. Their maps are added.
            obj.setHRfiles( 'ITM', '', 'calcThermal(0, r, 0, PcoatI, wI, 0.2, 0.17 )', 0.16, 0, 0, 'PcoatI', PcoatI, 'wI', wI );
            obj.setHRfiles( 'ETM', '', 'calcThermal(0, r, 0, PcoatE, wE, 0.2, 0.17 )', 0.16, 0, 0, 'PcoatE', PcoatE, 'wE', wE );

        end

        % function to calculate neam size
        % analuzeCavity builds the connections and 
        % setupModeBase calculates the mode base
        % After these calls, beam sizes are calculated internally.
        % usually, these are called automatically.
        function [wI, wE] = findBeamSize( obj )
            obj.analyzeCavity;
            obj.setupBaseMode;
            wI = obj.fields.fieldList( obj.fields.findNamedField( 'ITM-HR-o', -1 ) ).w;
            wE = obj.fields.fieldList( obj.fields.findNamedField( 'ETM-HR-o', -1 ) ).w;
        end

    end
end
