% function dat = HVThermal( elph, rlist, Ps, Pc, w, thickness, radius, varargin )
%  elph = 2 : dat(:,1) thermal surface defomraiton
%             dat(:,2) thermal lensing
%  elph = 0 : dat(:) thermal surface defomraiton
%  elph = 1 : dat(:) thermal lensing
%  elph =-1 : dat(:) thermal lensing except for the surface deformation
%
% This calculates the sufrace deformation and thermal lens of a test mass at radial points given in rlist
% for a given absorptions in the substrate (Ps) and in the coating (Pc) by a beam with size w
%
% Originally developed by Dr. Muzammil A. Arain
% when Muzammil was working in the LIGO Group, Department of Physics, University of Florida
%
% Program that calculates thermal aberration in a test mass with normal incident beam 
% based on Hello-Vinet theory
% 1) P. Hello and J.-Y. Vinet, ?Analytical models of thermal aberrations in massive mirrors 
%    heated by high power laser beams,? J. Phys. France 51, 1267?1282 (1990)
% 2) P. Hello and J.-Y. Vinet, ?Analytical models of transient thermoelastic deformations of 
%    mirrors heated by high power cw laser beams,? J. Phys. France 51, 2243?2261 (1990).
%
% dx=0.7/256; xlist=-127.5*dx:dx:127.5*dx; 
% Ps = 0.084; Pc = 0.425;
%
% July 28, 2015 HY : new option elph = -1 is added, which returns lens without the lens effect 
%                    due to surface formtion, i.e., lens - (n-1)surf
%

function dat= HVThermal( elph, rlist, Ps, Pc, w, thickness, radius, varargin )

ip = inputParser;
addParameter(ip, 'alpha',  5.1e-7,  @(x)isnumeric(x) && x>0); % Fused Silica coefficient of linear expansion
addParameter(ip, 'n',      1.45,    @(x)isnumeric(x) && x>0); % Refractive Index
addParameter(ip, 'thermo', 8.7E-6,  @(x)isnumeric(x));        % dn/dt of fused silica
addParameter(ip, 'kt',     1.38,    @(x)isnumeric(x) && x>0); % Thermal Conductivity
addParameter(ip, 'emiss',  0.91,    @(x)isnumeric(x) && x>0); % Emissivity
addParameter(ip, 'fac',    0.17,    @(x)isnumeric(x) && x>0); % external ambient temperature
addParameter(ip, 'T0', 	   300,     @(x)isnumeric(x) && x>0); %Fused Silica coefficient of linear expansion
ip.parse(varargin{:})

sbcons=5.67051E-8;  % Stefan-Boltzmann constant

tau=4*ip.Results.emiss*sbcons*ip.Results.T0^3*radius/ip.Results.kt;
zhik = getzhik( tau, 100 );
pk = getpk( zhik, radius, tau, w );

gak=zhik*thickness/(2*radius);

Ak=0.5./(zhik.*sinh(gak)+tau*cosh(gak));
Bk=0.5./(zhik.*cosh(gak)+tau*sinh(gak));

% if elph == -1, calculate both and subtract surface effect from lens
remove = elph == -1;
if remove
    elph = 2;
end

if elph == 2
    dat = zeros( length(rlist), 2 );
else
    dat = zeros( length(rlist), 1 );
end

%  Calculation of Phase Aberrations
if elph == 2 || elph == 0
    % for either case, it will be in dat(:,1)
	dat(:,1) = getThermoElsaticDefrom( rlist, Pc, Ps, ip.Results.alpha, ip.Results.kt, radius, tau, ...
    thickness, pk, zhik, gak, Ak, Bk, ip.Results.fac );
end

% Lens in the substrate
if elph == 2 || elph == 1
    % for either case, it will be in dat(:,elph)
    dat(:,elph) = getThermalPhaseDeform( rlist, Pc, Ps, ip.Results.alpha, ip.Results.kt, radius, tau, ...
    pk, zhik, gak, Ak, Bk, ip.Results.n, ip.Results.thermo );
end

if remove
    % remove the lens due to surface formation
    dat(:,1) = dat(:,2) - (ip.Results.n-1)*dat(:,1);
    % remove the data
    dat(:,2) = [];
end

return 

% 
% solution of z besselj(1,z) - tau besselj(0,z) = 0
% total of pmax zeros

function zlist = getzhik( tau, pmax )

zlist = zeros( 1, pmax );
for k = 1 : pmax
    zval0 = ( k - 1 + 0.25 ) * pi;
    delzval = 0.01;
    while delzval > 0.0001
        val0 = zval0 * besselj(1,zval0) - tau * besselj(0,zval0);
        zval = zval0 + delzval;
        valp = zval * besselj(1,zval) - tau * besselj(0,zval);
        zval = zval0 - delzval;
        valm = zval * besselj(1,zval) - tau * besselj(0,zval);
        a = (valp + valm)/2 - val0;
        b = (valp - valm)/2;
        if a == 0
            delzval = val0 / b * delzval;
        else
            if valp >= valm
                zsig = 1;
            else
                zsig = -1;
            end
            delzval = ( -b + zsig * sqrt( b*b - 4*a*val0) ) / (2*a) * delzval;
        end
        zval0 = zval0 + delzval;
        delzval = abs(delzval);
    end
    zlist(k) = zval0;
end
return

% 
% Dini series expansion of input gaussian beam

function pk = getpk( zhik, radius, tau, w )

pk = zeros( size(zhik) );
for k = 1 : length( zhik )
    zhikK = zhik(k);
    conv = zhikK / radius * w / sqrt(2);
    intLow = exp(-2*(radius/w)^2);
    Nbin = min( floor(zhikK/pi+1)*32, 128 ) * 4;
    delU = ( 1 - intLow ) / Nbin;
    sum = 1 + besselj(0, zhikK) + 4*besselj(0, conv*sqrt(-log(1 - delU)));
    u = 1;
    for jj = 2 : (Nbin/2)
        u = u - 2*delU;
        sum = sum + 2*besselj(0,conv*sqrt(-log(u))) + 4*besselj(0,conv*sqrt(-log(u-delU)));
    end
    pk(k) = (zhikK/radius/besselj(0,zhikK))^2/(zhikK^2+tau^2)/pi * sum / 3 * delU;
end
return


% 
% Surface deformation in m

function ucrc = getThermoElsaticDefrom( rlist, Pc, Ps, alpha, kt, radius, tau, ~, pk, zhik, gak, Ak, Bk, fac )

coeff=Pc*(1+fac)*alpha/kt;      %Coefficient for loss in coating
coeffcor=Pc*3*(1-fac)*0.5*alpha/kt;      %Saint Vinet Correction Coefficient
coeffs=Ps*(1+fac)*alpha/kt;

ucrc = zeros( size(rlist) );

for i = 1 : length(rlist)
    r=rlist(i);
    summation=radius^2*pk./zhik.*(Ak.*sinh(gak)+Bk.*cosh(gak)).*real(besselj(0,zhik*r/radius)-1);
    summat=(sum(summation));
    
    scor=pk.*Bk./(gak.^2).*(sinh(gak)./gak-cosh(gak)).*real(besselj(1,zhik)) * r^2;
    scorsum=coeffcor*sum(scor);

    summations=-radius^2*pk./(zhik.^2).*((sinh(gak))./(gak)).*(tau*Ak-(sinh(gak))./(gak+cosh(gak).*sinh(gak))).*real((besselj(0,zhik*r/radius))-1);
    summats=sum(summations);                

    ucrc(i)=coeff*summat+scorsum+coeffs*summats;    %Total Surface deformation including substrate and coating heating
end
ucrc=ucrc-max(ucrc);          % This is the variable to be saved for Surface Thermal Aberrations
return


% 
% thermal lens

function phit = getThermalPhaseDeform( rlist, Pc, Ps, alpha, kt, radius, tau, pk, zhik, gak, Ak, ~, n, thermo )

coeffsublens=Ps*(thermo+alpha*(n-1))/kt*radius^2;
coeffcolens=Pc*(thermo+alpha*(n-1))/kt*radius^2;

phist = zeros( size(rlist) );
phict = zeros( size(rlist) );
phit = zeros( size(rlist) );

for i = 1 : length(rlist)
    r = rlist(i);
    
    phissum=pk./(zhik.^2).*((-2*tau*Ak.*sinh(gak))./(gak)+1).*real((besselj(0,zhik*r/radius)));
    phisss=sum(phissum);
    phist(i)=(coeffsublens)*phisss;   %Phase Aberration due to substrate heating

    phicosum=pk./zhik.*(2*Ak.*sinh(gak)).*real(besselj(0,zhik*r/radius));
    phicoat=sum(phicosum);
    phict(i)=(coeffcolens)*phicoat;    %Phase Aberration due to coating heating
    phit(i)=phict(i)+phist(i);        %Total Phase Aberration
end

phit=phit-max(phit);      %  This is the variable to be saved for Substrate Thermal Aberrations

return
