%
% ModalModelTK : toolkit to calculate modal model related functions
%   needs ContantTK
%
% September 2, 2015
%   name convension is to use inv when in or out variables are inv
%
% September 1st, 2015
%   lam0 and kLam0 are defined in this class, not in ConstantTK.
%   For now, there is no default argument
%
% August 30, 2015
%   RinvW2zz0 and RinvWZ2RinvW to calulate quantities using R and w (and k)
%
% February 24, 2012
%   Consisten use of inv, all names are Xinv, like Rinv, qinv, etc
%   Remove redundant expressions
%
% August 5, 2010 
%   Use Rinv (=1/R) instead of R as arguments.
%   f number is removed, except RinvBack.
%
% June 24, 2010  Hiro Yamamoto  hiro@ligo.caltech.edu
%   first release
%

classdef MMTK
    
%     properties (Constant)
%         lam0 = 1.064e-6; % default CR laser wave length
%         kLam0 = 2*pi / MMTK.lam0;
%     end

    methods (Static)
        
        % effective curvature seen from back side of a mirror 
        % with refractive index n and internal f number invF
        % Rm > 0 for concave mirror
        % Use this for the second argument of FPparam to 
        % calculate parameters of a Michelson cavity
        function Rinv = RinvBack( Rinv, n, Finv )
            Rinv = - n * Rinv + Finv;
        end

        % modal paramters in a FP cavity defined by curvatures and the cavity length.
        % returns Rayleigh range and the front mirror z
        function [z, z0] = FPparam( ROC1inv, ROC2inv, Lcav )
            g1 = 1 - Lcav * ROC1inv;
            g2 = 1 - Lcav * ROC2inv;
            z = Lcav * g2.*(1 - g1)./(g1 + g2 - 2*g1.*g2);
            z0 = Lcav * sqrt( g1.*g2.*(1 - g1.*g2) ) ./ abs( g1 + g2 - 2*g1.*g2 );
        end
        
        % q of a FP cavity
        function q = FPparamQ( ROC1inv, ROC2inv, Lcav )
            [z, z0] = MMTK.FPparam( ROC1inv, ROC2inv, Lcav );
            q = z + 1i * z0;
        end
        
        % beam size on front mirror
        function w = w1FP( ROC1inv, ROC2inv, Lcav, k )
%             if ~exist( 'k', 'var' )
%                 k = MMTK.kLam0;
%             end
            [z, z0] = MMTK.FPparam( ROC1inv, ROC2inv, Lcav );
            w0 = sqrt(2*z0/k);
            w = w0*sqrt( 1 + (z/z0)^2 );
        end

        function RoC = wL2RoC( w, L, gPlusMin, k )
            % RoC = wL2RoC( w, L, gPlusMin, k )
            % RoC of a symmetric cavity which makes the beam size to be w,
            % gPlusMin is the choice of positive or negative g
%             if ~exist( 'k', 'var' )
%                 k = MMTK.kLam0;
%             end
%             if ~exist( 'plusMinus', 'var' )
%                 gPlusMin = -1;
%             end
            A = 4 * L / (k^2 * w^4);
            RoC = sqrt( 1 - A * L );
            if gPlusMin > 0
                RoC = ( 1 + RoC ) / A;
            else
                RoC = ( 1 - RoC ) / A;
            end
        end
        
        function eta = gouy00FP( ROC1inv, ROC2inv, Lcav )
        % eta = gouy00FP( ROC1inv, ROC2inv, Lcav )
        % gouy phase of the FP cavity
            [z, z0] = MMTK.FPparam( ROC1inv, ROC2inv, Lcav );
            eta = atan2( Lcav - z, z0 ) + atan2( z, z0 );
        end
        
        % gouy phase of a field from q = z + i z0 to q + L
        function eta = gouyL( q, L )
            eta = atan2( real(q)+L, imag(q) ) - atan2( real(q), imag(q) );
        end

% ABCD matrix toolkit
        
        % ABCD operation on q
        function qp = ABCD_q( M, q )
            qp = ( M(1,1) * q + M(1,2) ) ./ ( M(2,1) * q + M(2,2) );
        end
        
        % ABCD operation on 1/q
        function qpinv = ABCD_qinv( M, qinv )
            qpinv = ( M(2,1) + M(2,2)*qinv ) ./ ( M(1,1) + M(1,2)*qinv );
        end
        
        % when qInv' is given for qInv' = ABCD(qInv), qInv is calculated
        function qinv = ABCD_qinvpRev( M, qinvp )
            qinv = ( M(2,1) - M(1,1)*qinvp ) ./ ( M(1,2)*qinvp - M(2,2) );
        end
        
        % ABCD operation on (z, z0)
        function [zp, z0p] = ABCD_zz0( M, z, z0 )
            qp = ABCD_q( M, z + 1i*z0 );
            zp = real( qp );
            z0p = imag( qp );
        end
        
        % ABCD operation on (1/R and w)
        function [Rpinv, wp] = ABCD_rw( M, Rinv, w, k )
%             if ~exist( 'k', 'var' )
%                 k = MMTK.kLam0;
%             end
            [Rpinv, wp] = MMTK.qinv2RinvW( MMTK.ABCD_qinv( M, Rinv - 1i*k*2/w^2 ), k );
        end
        
        % propagation through a substrate with refractive index n
        function M = Prop_ABCD( L, n )
            if ~exist('n', 'var')
                n = 1;
            end
            M = [ 1, L/n; 0, 1 ];
        end
        
        % lens specified by f number
        function M = LensF_ABCD( finv )
            M = [ 1, 0; -finv, 1 ];
        end
        
        % lens through a substrate with refractive index n and 
        % one side ROCinv ( positive for concave )
        function M = LensR_ABCD( Rinv, n )
            M = MMTK.LensF_ABCD( -(n-1)*Rinv );
        end
        
        % reflection with angle of incidence
        % plane = 0 : tangential (in the plane of incidence)
        %       = 1 : sagittal (perpendicular to plane of incidence)
        function M = Refl_ABCD( Rinv, theta, plane )
            if ~exist('plane', 'var')
                plane = 0;
            end
            if ~exist('theta', 'var')
                theta = 0;
            end
            if plane == 0
                cosCor = cos( theta );
            else
                cosCor = 1/cos( theta );
            end
            M = [ 1, 0; -2*Rinv/cosCor, 1];
        end
        
        % reflection from the AR side, normal incident case
        % RoC looks like -R/n
        % TO BE EXPANDED FOR ARBITRARY ANGLE
        function M = ReflAR_ABCD( Rinv, n, d )
            if ~exist( 'd', 'var' )
                d = 0;
            end
            M = [ 1, 0; 2*n*Rinv, 1];
            if d ~= 0
                M = MMTK.Prop_ABCD( d, n ) * M * MMTK.Prop_ABCD( d, n );
            end
        end
        
        % eigen value of a cavity defined by M
        function qEigen = EigenMode_ABCD( M )
            z = (M(1,1) - M(2,2)) / (2*M(2,1));
            z0Sq = -(M(1,1)-M(2,2))^2 - 4*M(1,2)*M(2,1);
            if z0Sq > 0
                z0 = sqrt( z0Sq ) / abs( 2*M(2,1) );
            else
                z0 = NaN;
            end
            qEigen = z + 1i * z0;
        end
        
        % constucting M by reflection and transmission
        % if number of argument is odd, the arguments will be for one way
        % even case : r1 p12 r2 p23 r3 p32 r2 p21
        % odd case  : r1 p12 r2 p23 r3 [p23] [r2] [p12]
        function M = ReflProp_ABCD( varargin )
            if mod(nargin,2) == 0
                M = MMTK.Refl_ABCD( varargin{1} ) * MMTK.Prop_ABCD( varargin{2} );
                if nargin > 2
                    M = M * MMTK.ReflProp_ABCD( varargin{3:end} );
                end
            else
                if nargin == 1
%                    varargin{1}
                    M = MMTK.Refl_ABCD( varargin{1} );
                else
                    M = MMTK.ReflProp_ABCD( varargin{1:end-1} ) * ...
                        MMTK.ReflProp_ABCD( varargin{end:-1:2} );
                end
            end
        end
        
        % adjust length, which is negative, to make the RoC(qOut) = RoC1
        % when qIn propagates through the cavity defined
        % inputs : RoC1 L12 RoC2 L23 ... L, qIn, dL
        % qOut = ABCD( RoC1, L12, ..., L ) * qIn
        function [len0, qOut] = AdjustModeMatch( varargin )
            adjust = find( [varargin{2:2:end-2}] < 0 ) * 2;
            L0 = -varargin{adjust};
            dl0 = varargin{nargin};
            dl = dl0;
            len0 = L0;
            lenMin = L0 - dl;
            lenMax = L0 + dl;
            invRoC = varargin{1};
            qIn = varargin{end-1};

            % remove the reflection by the mirror at qOut
            varargin{1} = 0;
            
            % look for length to make qOut to match with RoC1
            invRoCOut = zeros(1,2);
            dinvR = zeros(1,2);
            while (dl/L0) > 1e-6
                len(1) = len0 - dl;
                len(2) = len0 + dl;
                for iLen = 1:2
                    varargin{adjust} = len(iLen);
                    M = MMTK.ReflProp_ABCD( varargin{1:end-2} );
                    qOut = MMTK.ABCD_q( M, qIn );
                    invRoCOut(iLen) = real(1/qOut);
                    dinvR(iLen) = invRoCOut(iLen) - invRoC;
                end
                % at -dl => dLen(1), at +dl => dLen(2)
                dinvR12 = dinvR(1) - dinvR(2);
                % dR/R is small enough
                if abs( dinvR12 / invRoC ) < 1e-6
                    break;
                end
                % expected 0 crossing is len0 + off*dl
                dl = ( dinvR(1) + dinvR(2) ) / dinvR12 * dl;
                len0 = len0 + dl;
                dl = abs(dl);
                if len0 < lenMin
                    len0 = lenMin;
                    break;
                elseif len0 > lenMax
                    len0 = lenMax;
                    break;
                end
                dl = min( dl, dl0 );
            end
            if nargout > 1
                varargin{adjust} = len0;
                M = MMTK.ReflProp_ABCD( varargin{1:end-2} );
                qOut = MMTK.ABCD_q( M, qIn );
            end
        end

% conversion among various forms
        function [z, z0] = RinvW2zz0( Rinv, w, lam )
            % [z, z0] = RinvW2zz0( Rinv, w, lam )
            % calculete (z,z0) from Rinv and w
%             if ~exist( 'lam', 'var' )
%                 lam = MMTK.lam0;
%             end
            [z, z0] = MMTK.q2zz0( 1/MMTK.RinvW2Qinv( Rinv, w, lam ) );
        end

        function [RinvZ, wZ] = RinvWZ2RinvW( Rinv, w, z, lam, k )
            % [z, z0] = [RinvZ, wZ] = RWZ2RW( Rinv, w, z, lam )
            % calculete (Rinv, w) at z from (Rinv, w) at z=0
%             if ~exist( 'lam', 'var' )
%                 lam = MMTK.lam0;
%             end
            q = 1 / MMTK.RinvW2Qinv( Rinv, w, lam ) + z;
            [RinvZ, wZ] = MMTK.qinv2RinvW( 1./q, k );
        end
    
        function qinv = RinvW2Qinv( Rinv, w, lam )
            %	function qinv = RinvW2Qinv( Rinv, w, lam )
            %   Rinv, w to qInv
%             if ~exist( 'lam', 'var' )
%                 lam = MMTK.lam0;
%             end
            qinv = Rinv - 1i*lam / pi / w^2;
        end
        
        % q to (z, z0)
        function [z, z0] = q2zz0( q )
            z = real(q);
            z0 = imag(q);
        end
        
        function [Rinv, w] = qinv2RinvW( qinv, k )
            %   function [Rinv, w] = qinv2RinvW( qinv, k )
            % q to 1/R and w
            Rinv = real(qinv);
            if nargout == 2
                w = MMTK.qinv2W(qinv, k);
            end
        end
        
        % w from q
        function w = qinv2W( qinv, k )
%             if ~exist( 'k', 'var' )
%                 lam = MMTK.kLam0;
%             end
            w = sqrt( -2./(k*imag(qinv)) );
        end
        
        % reverse direction ( q or invq will work )
        % q : z + i z0 => -z + i z0 or 
        % invq : 1/R - i lambda/pi 1/w^2 => -1/R - i lambda/pi 1/w^2
        function qp = reverseDir( q )
            qp = -real( q ) + 1i*imag( q );
        end
    end
end
