%
% MirrorObj : basec lass of mirror object, 
%             mirror2x2 (normal incident) and mirror4x4 (finite incident angle) are derived from this
%
%   invRoCAR should be set explicitly by hand like
%       mirID = obj.addMirror( ... )
%       obj.optics.optList(mirID).invRoCAR = 1/RoCofARofThisMirror
%
%   calculate reflection by a mirror, in and out are both internal format
%   inField is flipped left-right
%       outField = refl( obj, inPortIndex, inField )
%
%   calculate transmission through a mirror, in and out are both internal format
%   No field is flipped left-right
%       outField = trans( obj, inPortIndex, inField )
%
% September 2, 2015
%    Using new OpticObj structure with ifo property, mostly
%    changing ConstantTK.kLam0 to obj.ifo.fields.kCR
%
% July 8th, 2015
%    delSphPar added to handle spehrical mirror surface more accurately
%       parabalic case : r^2 / 2 RoC
%       spherical case : r^2 / 2 RoC x 2 / ( 1 + sqrt(1 - rsq / RoC^2) )
%
% November 26, 2014
%    applyDisplacement when applied to reflection map itself, the magnitude is compromised.
%    It is applied to the phase and mask.
% 
% November 15, 2014
%   applyDisplacement is to displace object by (optX0, optY0). 
%   buildMirFFTmaps is modified so that mirrors are displaced.
%   For convenience, phase is calculated which take care of phases of the field.
%   This is moved in the opposite direction to be combined with the original mirror.
%   optTheta and optPhi are implemenetd in buildMirFFTmaps for tilt
%
% October 15, 2014
%   trans() is modified so that, if transMatch(inPortIndex) == inPortIndex, trans acts as refl for back scattering
%
% August 5th, 2014
%   removeCurvature option is added to Mirror2x2
%   doRemoveTilt is replaced by doRemoveTiltCurvature and curvature mismatch is corrected as well
%
% October 22, 2013
% September 12, 2013
%
classdef MirrorObj < OpticObj

    properties (SetObservable)
        nRef = ConstantTK.nSubsSiO2;
    end
    
    properties (Constant)
        inputNames0x0 = { 'invRoC', 'Aperture', 'Thickness', 'virtualAperture', ...
            'T', 'Lhr', 'Lar', 'optTheta', 'optPhi', 'optX0', 'optY0', ...
            'isSpherical', 'invFocal', 'resolution', 'WfftUser', 'nRef' };
    end
    
    methods (Static)
        function tagList = getInputNames
            tagList = MirrorObj.inputNames0x0;
        end
    end

    properties (Dependent)
        cosAngInSub
        angInSub
        beamShift
    end
    
    properties
        % incident angle, default is normal incident
        incAngle = 0;
        
        % substrate
        Thickness = 0;
        
        % displacement / orientation
        %   mech.[x,y,z,tX,tY,tZ]
        
        % optical quantities
        t = 0;
        rHR = -1;
        Rhr0 = 1; % This is used for the round trip calculation. Pure reflectance without loss
        rAR = 1;
        
        % tile definition using eulter angle
        optTheta = 0;   % rotation around y axis after phi roration
        optPhi = 0;     % rotation around z axis
        optDCurv = 0;   % curvature correction to nullify curvature mismatch
        removeTilt = false;  % user flag if alighnment correction is to be done
        removeCurvature = false; % user flag if curvature mismatch correction is to be dobe
        
        invFocal = 0; % focal length (lens = -r^2/2*invF)
        
        % These will be used in mirror2x2 and in mirror4x4
        % mirror4x4 can be specified either by this HR surface (MC end mirror or RC folding mirrors)
        % or by 4 maps, 2 transmission and 2 reflection
        
        % HR surface specification
        HRfiles     % by data file
        % transmisson specification
        TRfiles     % by data file
        % AR surface specification
        % This is used mainly for BS spec to generate mocked up maps
        % For now, it is not used for Mirror2x2
        ARfiles     % by data file
        
        % each are two diemtional, (1) is for input to HR side, (2) is for input to AR side
        reflFFT
        transFFT
        
        % these are used fot debugging
        % to save memory, these can be temporal variable
        RPYmap
        RSXmap
        TPXmap
        TSYmap

        % parabolic or spherical mirrors
        isSpherical = true
        
        % refractiove index of HeNe which is used used for map measuremts by Zygo
        nRefMeasure = 1.457
        
        % after all maps are built, this function is called
        % refl are trans are cell(1,4) for mirror4x4 and cell(1,2) for mirror2x2,
        % and each are N/2xN/w maps to be applied to convert in(i) -> out
        % [refl, trans] = modMap( refl, trans, x, y )
        % store handle of this function if necessary
        modMap
        modMapVar = '' % this is passed as the last argument, use it like setting variables
        
        % variables defined when setMisc is called
        % it should be 2N cells, 'name1', val1, 'name2', var2, ...
        variables
    end
    
    methods
        function obj = MirrorObj( ifo, nIn, nOut, name, spec )
            
            obj = obj@OpticObj( ifo, name, nIn, nOut);

%            obj.isMirrorObj = true;
            
            spec = obj.defineRTL( spec );
            
            [A, spec] = UtilTK.extractVal( 'Aperture', spec );
            if ~isempty(A)
                obj.ApertureDef = A;
            end
            
            UtilTK.updateProperties( obj, obj.inputNames0x0, spec );
            
            addlistener(obj, 'nRef', 'PostSet', @obj.updateNdep);
            
            obj.HRfiles = dataFileManager('HRfiles');
            obj.TRfiles = dataFileManager('TRfiles');
            obj.ARfiles = dataFileManager('ARfiles');

            obj.variables = cell(0);
        end
        
        function map = getHRmap( obj, removeRoC )
            if ~exist( 'removeRoC', 'var' )
                removeRoC = true;
            end
            map = obj.RPYmap/2;
            if removeRoC
                [x,y] = obj.getXYR;
                map = map - (x.^2 + y.^2)*obj.invRoC/2;
            end
        end

        function [RoC, piston] = getHRRoC( obj, radius )
            [xl,yl] = obj.getXYR;
            [~, coef] = zernikeCoef( obj.RPYmap/2, xl, yl, radius );
            % coef:(1) piston, (2) tilt x, (3) tilt y, (4) power
            RoC = radius^2/(4*coef(4));
            piston = coef(1);
        end
        
        function loss = mirrorLoss( obj, inPortID )
            % loss = mirrorLoss( obj, inPortID )
            %   loss = 1 - ( reflected power + transmitted power ) / input power
            loss = nan;
            for ip = 1 : length( obj.inPorts )
                if obj.inPorts(ip) == inPortID
                    obj.ifo.fields.calcIfNecessary( inPortID );
                    inPower = obj.ifo.fields.fieldList(inPortID).calcSumIn(obj.Aperture/2);
                    if inPower == 0
                        return;
                    end
                    outPower = 0;
                    if length( obj.reflMatch ) >= ip
                        reflPortID = obj.outPorts( obj.reflMatch(ip) );
                        if reflPortID > 0
                            EFFTsave = obj.ifo.fields.fieldList(reflPortID).EFFT;
                            obj.refl( ip, obj.ifo.fields.fieldList(inPortID).EFFT );
                            outPower = obj.ifo.fields.fieldList(reflPortID).calcPower;
                            obj.ifo.fields.fieldList(reflPortID).EFFT = EFFTsave;
                        end
                    end
                    if length( obj.transMatch ) >= ip
                        transPortID = obj.outPorts( obj.transMatch(ip) );
                        if transPortID > 0
                            EFFTsave = obj.ifo.fields.fieldList(transPortID).EFFT;
                            obj.trans( ip, obj.ifo.fields.fieldList(inPortID).EFFT );
                            outPower = outPower + obj.ifo.fields.fieldList(transPortID).calcPower;
                            obj.ifo.fields.fieldList(transPortID).EFFT = EFFTsave;
                        end
                    end
                    loss = 1 - outPower / inPower;
                    return;
                end
            end
        end
        
        function spec = defineRTL( obj, spec )
        % calculate r and t from specification
        
            % T is power transmittance of HR surface
            [T, spec] = UtilTK.extractVal( 'T', spec, 0 );
            [Lhr, spec] = UtilTK.extractVal( 'Lhr', spec, 0 );
            [Lar, spec] = UtilTK.extractVal( 'Lar', spec, 0 );
            
            obj.rHR = 1 - T - Lhr;
            if obj.rHR < 0
                error('T + Loss on HR > 1');
            end
            % 1-Lar is transmittance of AR surface
            % (1-Lar)*Rhr = (1-Lar)*(1-T-Lhr) ~= 1 - Lar - T
            obj.rAR = (1-Lar)*obj.rHR;
            if obj.rAR < 0
                error('T + Loss on AR > 1');
            end
            % T is treated like transmittance of HR surface
            obj.t = T * (1 - Lar);
            if obj.t < 0
                obj.t = 0;
            end
            obj.t = sqrt(obj.t);
            obj.rHR = -sqrt(obj.rHR);
            obj.Rhr0 = 1 - T; % no loss component
            obj.rAR = sqrt(obj.rAR);
        end
                
        %% BS geometrical properties
        
        % angle in the substrate
        function inAng = get.angInSub( obj )
            inAng = asin( sin(obj.incAngle) / obj.nRef );
        end
        
        % cos angle in the substrate
        function cosAng = get.cosAngInSub( obj )
            cosAng = sqrt ( 1 - (sin(obj.incAngle) / obj.nRef)^2 );
        end
        
        % offset of the beam in x of points entering to one side and leaving on the other
        function d = get.beamShift( obj )
            d = obj.Thickness * tan( acos( obj.cosAngInSub ) );
        end
        
        % physical length to go through a substrate
        % n = 1, 0, -1 : length * nRef^n
        % correctOffcentering = true : include the effect of the beam location on the
        % AR side. Usually, the distance between mirrors are defined
        % as the distance of centers of two mirrors. When the beam is
        % off-centers on the AR side, the length becomes shorter
        function d = thicknessInSubs( obj, n, correctOffcentering )
            % d is the geometrical length of the ray in the substrate
            d = obj.Thickness / obj.cosAngInSub;
            if exist( 'n', 'var' )
                switch n
                    case -1
                        d = d / obj.nRef;
                    case 1
                        d = d * obj.nRef;
                end
            end
            if exist( 'correctOffcentering', 'var' )
                if correctOffcentering && (obj.incAngle ~= 0)
                    % using the geometrical length, calculate the
                    % geometrical change due to the off-centering on AR
                    d = d - obj.Thickness *( cos(obj.incAngle - obj.angInSub) / obj.cosAngInSub - cos(obj.incAngle) );
                end
            end
        end
        
        %% clear apertures of four maps
        function Amask = calcMask( obj, kind )
            if obj.ApertureDef < 0 || kind == 0
                % normal incident case or mirror size is very large
                Amask = obj.ifo.ffts.zeroMat;
                Amask( obj.getRsq <= (obj.Aperture^2/4) ) = 1;
            else
                % oval of one BS surface viewed by one beam
                % some centered, some offset, and one full path is a product of two ovales
                oval = @(xoff) ( ( (obj.ifo.ffts.Xm * obj.Wfft - cos(obj.incAngle)*xoff ) / cos(obj.incAngle) ).^2 + ...
                     (obj.ifo.ffts.Yn * obj.Wfft).^2 ) <= (obj.Aperture^2/4);
                switch kind
                    case 1
                        % BS ref from PRM to Y arm
                        Amask = oval( 0 );
                    case 2
                        % BS ref from SRM to X arm
                        Amask = oval( obj.beamShift ) & oval( -obj.beamShift );
                    case 3
                        % BS trans from PRM to X arm, +x is clipped
                        Amask = oval( -obj.beamShift ) & oval( 0 );
                    case 4
                        % BS trans from Y arm to SRM, -x is clipped
                        Amask = oval( obj.beamShift ) & oval( 0 );
                end
            end
            Amask = im2double( Amask );
        end     
        
        %% function [ RPY, RSX, TPX, TSY, specified ] = prepare2x2Like( obj )
        %
        % this creates maps based on three 2D maps, HR surface, transmission and
        % AR surface maps.
        % The transmission map with finite incident angle is built assuming
        % the substrate non-uniformity is deapth independent, and may not be accurate
        % To handle realistic mirrors with internal structure,
        % specify 2 reflection and 2 transmission maps
        %
        % if no files are specified, it is assumed perfect        

        function [ RPY, RSX, TPX, TSY, specified ] = prepare2x2Like( obj )
            
            specified = 0;
            
            [hrmap, invRoCHR] = obj.HRfiles.loadMaps( obj );
            if isempty( hrmap )
                % if no file is specified
                hrmap = zeros(size(obj.getXYR));
                invRoCHR = obj.invRoC;
            else
                % if any file is specified and yet, RoC is not defined, use the original invRoC
                specified = specified + 1;
                if invRoCHR == 0
                    invRoCHR = obj.invRoC;
                end
            end
            % based on the curvature and incident angle,
            % calculate the surface figure and position dependent reflection angle
            % HR surface = RPYmap + s, s ~ -r^2/RoC
            [s, cosAlpha] = obj.evalAberration( invRoCHR );    
            RPY = 2 * hrmap .* cosAlpha - s*cos(obj.incAngle);
            % hrmap has the curvature included
%            hrmap = hrmap + obj.getRsq/2*invRoCHR*cos(obj.incAngle);
            hrmap = hrmap + MirrorObj.delSphPar( obj.getRsq, invRoCHR, obj.isSpherical )*cos(obj.incAngle);
            
            % transmission map and AR side reflection maps
            % SPTMAP has curvature of HR side subtracted, or HR side curvature is the reference
            % This is true for BS (normal incident) and ITM
            [sptmap, invRoCTransMap]  = obj.TRfiles.loadMaps( obj );
            if isempty( sptmap )
                sptmap = zeros(size(obj.getXYR));
            else
                specified = specified + 2;
                sptmap = sptmap + obj.getRsq/2*invRoCTransMap;
            end
            
            [armap, invRoCARMap] = obj.ARfiles.loadMaps( obj );
            if isempty( armap )
                armap = zeros(size(obj.getXYR));
            else
                specified = specified + 4;
                % AR side reflection is not important, so no s
                % in case there is AR side curvature...
%                armap = armap + obj.getRsq/2*invRoCARMap;
                armap = armap + MirrorObj.delSphPar( obj.getRsq, invRoCARMap, obj.isSpherical );
            end
            
            % compose the 4 maps for BS out of three 2D maps,
            % HR and AR reflection + transmission maps
            
            if obj.incAngle ~= 0
                % compose additional three maps using hrmap, tmap and armap
                [RSX, TPX, TSY] = obj.tiltMaps( hrmap, sptmap, armap );

                [x0, y0] = obj.getXYR;
                x1 = x0*cos(obj.incAngle);
                RPY = interp2( x1, y0, RPY, x0, y0 );
                RSX = interp2( x1, y0, RSX, x0, y0 );
                TPX = interp2( x1, y0, TPX, x0, y0 );
                TSY = interp2( x1, y0, TSY, x0, y0 );
            else
                % RPY = 2 * hrmap .* cosAlpha - s*cos(obj.incAngle); calculated when hrmap is provided
                % this conversion is done properly in tiltMaps
                sptmap = sptmap * obj.nRef/ obj.nRefMeasure;
                RSX = 2 * ( sptmap - hrmap*obj.nRef );
                TPX = sptmap - (obj.nRef-1)*hrmap;
                TSY = TPX;
            end
        end
        
        %% [RPY, RSX, TPX, TSY ] = buildDefault( obj )
        % build maps when nothing is spcified using parameters in addMirror
        % this is for normal incident case, or very close to it
        function [RPY, RSX, TPX, TSY ] = buildDefault( obj )
            
            % HR side reflection
            hrmap = obj.getRsq/2*obj.invRoC;
        	s = obj.evalAberration( obj.invRoC );    
            
            % transmission map and AR side reflection maps
            sptmap = obj.getRsq/2*obj.invFocal;

            % AR side spherical map
            armap = obj.getRsq/2*obj.invRoCAR;
            
            RPY = -s*cos(obj.incAngle);
            RSX = 2 * ( sptmap - hrmap*obj.nRef - armap*(obj.nRef-1) );
            TPX = sptmap - (obj.nRef-1)*(hrmap + armap);
            TSY = TPX;
        end
        
        %%
        % Compose BS maps out of 2D transmission maps.
        % The transmission map with finite incident angle is built assuming
        % that the substrate non-uniformity is deapth independent.
        % Clear apertures for a finite size BS need to be multiplied to get correct maps,
        % I.e., maps are as large as data files can be applied.
        %
        % all quantities are calculated using optical path length.
        % At the end of the function, signs are flipped to conform to the Zygo measurement convension
        %
        function [RSX, TPX, TSY] = ...
                tiltMaps( obj, hrmap, sptmap, armap )
                        
            % definition of the orientation is different between hrmap and (sptmap, armap)
            % unify to hrmap. So all are facing HR out
            sptmap = fliplr(sptmap);
            armap = fliplr(armap);
            
            % shift of the beam location from oneside to another, in units of the FFT pixel
            nOffset = obj.beamShift / obj.getDXDY;
            
            % substrate optical thickness per unit physical length
            % assmuned the optical thickness is proportinally scaled by the ratio of
            % the nominal refractive index from HeNe to NdYaG
            % data / d0 (value in unit optical path length ) * D (optical thickness of each slice)
            % = data / d0 * d0 / cos(angInSub) / nOffset = data / cos / nOffset
            %
            % sptmap is genative of optical path length
            mapSub = ...
                ( -sptmap - (obj.nRefMeasure-1)*armap - obj.nRefMeasure*hrmap ) ...
                * obj.nRef / obj.nRefMeasure / obj.cosAngInSub / nOffset;
            
            % transmission maps from HR side to AR, TPX toward X Arm, TSY toward SRC
            TPX = zeros(size(mapSub));
            TSY = zeros(size(mapSub));
            
            % sum up layers with optical length D = d0 / cos(angInSub) / nOffset
            for in = 1 : ceil( nOffset )
                if in == ceil(nOffset)
                    weight = nOffset - floor(nOffset);
                else
                    weight = 1;
                end
                TPX(:,in:end) = TPX(:,in:end) + mapSub(:,1:end-in+1) * weight;
                TSY(:,1:end-in+1) = TSY(:,1:end-in+1) + mapSub(:,in:end) * weight;
            end
            
            [xl,yl] = obj.getXYR;
            
            % for TPX case, beam at the center on HR goes to -beamShift (facing to out from HR) on AR 
            % i.e., TPX(:,center) uses ARMAP(:,center-beamShift)
            if max(max(abs(armap))) == 0
                armapShift = zeros( size( TPX ) );
            else
                armapShift = interp2( xl, yl, armap, xl-obj.beamShift, yl );
            end
            TPX = TPX + (obj.nRef/obj.cosAngInSub - 1/cos(obj.incAngle)) * armapShift ...
                + obj.nRef/obj.cosAngInSub * hrmap;
            
            % for TSY case, beam at the center on HR goes to +beamShift on AR
            if max(max(abs(armap))) == 0
                armapShift = zeros( size( TPX ) );
            else
                armapShift = interp2( xl, yl, armap, xl+obj.beamShift, yl );    
            end
            TSY = TSY + (obj.nRef/obj.cosAngInSub - 1/cos(obj.incAngle)) * armapShift ...
                + obj.nRef/obj.cosAngInSub * hrmap;
            
            % I got clazy about the sign
            RSX = TPX + TSY;
            
            TPX = TPX - hrmap/cos(obj.incAngle);
            TSY = TSY - hrmap/cos(obj.incAngle);

            % TPX is a map measued from PRC, which is opposite to all other cases
            TPX = fliplr(TPX);
            
            % flip the sign to change the convension from optical path length to Zygo measurement convension
            RSX = -RSX;
            TPX = -TPX;
            TSY = -TSY;
        end
        
        %% build reflFFT and transFFT from RPY, RSX, TPX and TSY maps
        %
        % 1) HR side reflection
        %    in : PRC -> out : Y arm and in : Y arm -> out : PRC
        % 2) AR side reflection
        %    in : X arm -> out SRC and in : SRC -> out X arm
        % 3) transmission between PRC and X arm (HR to AR)
        %    For PRC to X, +x is clipped
        % 4) transmission from SRC to Y arm (AR to HR)
        %    For SRC to y, +x is clipped, so mask 3, same as PRC to x

        % maskID : Amask is caluclated by obj.calcMask( masiID(1:4) ), 
        %         for BS [1,2,3,3], 
        %         for mirror [0,0,0,0]
        % sides : for BS [1,2;3,4;1,3;2,4]
        %         for mirror [1;2;1;2]
        function buildMirFFTmaps( obj, masks, sides )
                                        
            [xMir, yMir, r2Mir] = obj.getXYR;
            r2Mir = r2Mir.^2;
            if ~obj.removeTilt && obj.optTheta ~= 0
                Cx = cos( obj.optPhi )*sin( obj.optTheta );
                Cy = sin( obj.optPhi )*sin( obj.optTheta );
                HRTiltCor = -2 * (Cx * xMir + Cy * yMir);
            else
                HRTiltCor = 0;
            end
            
            % HR side reflection
            % in PRC -> out Y arm, in Y arm -> out PRC
            Amask = obj.calcMask(masks(1));
            obj.RPYmap = obj.RPYmap .* Amask;
            for iside = sides(1,:)
                inID = obj.inPorts(iside);
                outID = obj.outPorts( obj.reflMatch(iside) );
                del = obj.ifo.fields.getPhiCorr(outID) - obj.ifo.fields.getPhiCorr(inID);
                % s is the change of reflection length, ~ -1/R, twice of hight x -1
                % RPYmap is height*2*cos
                phase = obj.applyDisplacement( HRTiltCor + obj.RPYmap, obj.reflMatch(iside) ) + del / 2 * r2Mir;

                % map is defined seeing from PRC
                % +x affects on Ein(-x) for both direction, so no flip
                obj.reflFFT{iside} = obj.rHR * exp( 1i * obj.ifo.fields.kCR * phase ) .* obj.applyDisplacement( Amask, obj.reflMatch(iside) );
            end
            
            % AR side reflection
            % in X arm -> out SRC, in SRC -> out X arm
            Amask = obj.calcMask(masks(2));
            obj.RSXmap = obj.RSXmap .* Amask;
            for iside = sides(2,:)
                inID = obj.inPorts(iside);
                outID = obj.outPorts( obj.reflMatch(iside) );
                % invRoCHR is the HR side curvature, and map is defined relative to that
                del = obj.ifo.fields.getPhiCorr(outID) - obj.ifo.fields.getPhiCorr(inID);
                % RSXmap is the pathlength from AR side to HR side and back to AR
                % Opposite to RPYmap
                % I got almost crazy about the sign
                phase = obj.applyDisplacement( -fliplr(HRTiltCor) + obj.RSXmap, obj.reflMatch(iside) ) + del / 2 * r2Mir;
                % map is defined seeing from SRC
                % +x affects on Ein(-x) for both direction, so no flip
                obj.reflFFT{iside} = obj.rAR * exp( 1i * obj.ifo.fields.kCR * phase ) .* obj.applyDisplacement( Amask, obj.reflMatch(iside) );
            end
            
            % transmission between PRC and X arm (HR to AR)
            % For PRC to X, +x is clipped
            Amask = obj.calcMask(masks(3));
            obj.TPXmap = obj.TPXmap .* Amask;
            for iside = sides(3,:)
                % iside 1 : PRC -> X arm
                % iside 3 : X arm -> PRC
                inID = obj.inPorts(iside);
                outID = obj.outPorts( obj.transMatch(iside) );
                del = obj.ifo.fields.getPhiCorr(outID) - obj.ifo.fields.getPhiCorr(inID);
                phase = obj.applyDisplacement( obj.TPXmap, obj.transMatch(iside) ) + del / 2 * r2Mir;
                % tmap is defined using axis defined by +z pointing out of HR
                % For PRC to X arm, outgoing field and map have opposite x direction
                obj.transFFT{iside} = obj.t * exp( 1i * obj.ifo.fields.kCR * phase ) .* obj.applyDisplacement( Amask, obj.transMatch(iside) );
                if iside == 3
                    obj.transFFT{iside} = fliplr( obj.transFFT{iside} );
                end
            end
            
            % transmission from SRC to Y arm (AR to HR)
            % For SRC to y, +x is clipped, so mask 3, same as PRC to x
            Amask = obj.calcMask(masks(4));
            obj.TSYmap = obj.TSYmap .* Amask;  % this is for debugging
            for iside = sides(4,:)
                % iside 2 : Y arm -> SRC
                % iside 4 : SRC -> Y arm
                inID = obj.inPorts(iside);
                outID = obj.outPorts( obj.transMatch(iside) );
                del = obj.ifo.fields.getPhiCorr(outID) - obj.ifo.fields.getPhiCorr(inID);
                phase = obj.applyDisplacement( obj.TSYmap, obj.transMatch(iside) ) + del / 2 * r2Mir;
                % hrmap and tmap are defined using axis defined by +z pointing out of HR
                % For Y arm to SRC, outgoing field and map have opposite x direction
                obj.transFFT{iside} = obj.t * exp( 1i * obj.ifo.fields.kCR * phase ) .* obj.applyDisplacement( Amask, obj.transMatch(iside) );
                if iside == 2
                    obj.transFFT{iside} = fliplr( obj.transFFT{iside} );
                end
            end
            
        end
        
         %% displace all reflFFT and transFFT maps by optX0 and optY0
         % This is used to shift the beam power term. That moves the beam opposite direction
        function newmap = applyDisplacement( obj, map, outIndex )
            if obj.optX0 == 0 && obj.optY0 == 0
                newmap = map;
                return;
            end

            [x,y] = obj.getXYR;
            if obj.isHRsideIndex( outIndex )
                xSign = 1;
            else
                xSign = -1;
            end
            newmap = interp2( x, y, map, x-obj.optX0*xSign, y-obj.optY0 );
        end

        %% distance change on reflection
        % s ~= -(x^2/cos(theta) + y^2*cos(theta))/R
        % cosAlpha ~= cos(incAngle)
        function [s, cosAlpha] = evalAberration( obj, inVRoCMap )
             % reflection on the HR side
            [x,y] = obj.getXYR;
            if obj.isSpherical
                [s, cosAlpha] = MirrorObj.Spherical( inVRoCMap, x, y, obj.incAngle );
            else
                [s, cosAlpha] = MirrorObj.Parabolic( inVRoCMap, x, y, obj.incAngle );
            end
        end
        
        %% calculate reflection by a mirror, in and out are both internal format
        % outField = reflFFT * fliplr( inField )
        % out( +x ) = reflFFT( +x ) * inField( -x )
        function outField = refl( obj, inPortIndex, inField )
            if nargout == 0
                outID = obj.outPorts(obj.reflMatch(inPortIndex));
                obj.ifo.fields.fieldList(outID).EFFT = obj.reflInOut( obj.reflFFT{inPortIndex}, inField );
%                obj.ifo.fields.fieldList(outID).status = -1; 
            else
                outField = obj.reflInOut( obj.reflFFT{inPortIndex}, inField );
            end
        end
        
        %% calculate transmission through a mirror, in and out are both internal format
        % outField = transFFT * inField
        % out( +x ) = reflFFT( +x ) * inField( +x )
        %
        % if transMatch(inPortIndex) == inPortIndex, this is back scattering and reflection formula is applied
        function outField = trans( obj, inPortIndex, inField )
            if nargout == 0
                outID = obj.outPorts(obj.transMatch(inPortIndex));
                if obj.transMatch(inPortIndex) == inPortIndex
                    % inport == outport is back reflection
                    obj.ifo.fields.fieldList(outID).EFFT = obj.reflInOut( obj.transFFT{inPortIndex}, inField );
                else
                    % normal transmission
                    obj.ifo.fields.fieldList(outID).EFFT = obj.transInOut( obj.transFFT{inPortIndex}, inField );
                end
%                obj.ifo.fields.fieldList(outID).status = -1;
            else
                if obj.transMatch(inPortIndex) == inPortIndex
                    % inport == outport is back reflection
                    outField = obj.reflInOut( obj.transFFT{inPortIndex}, inField );
                else
                    % normal transmission
                    outField = obj.transInOut( obj.transFFT{inPortIndex}, inField );
                end
            end
        end

        %% about this mirror
        function info = printInfo( obj )
            info = printInfo@OpticObj( obj );
            loss = 1-obj.rHR^2-obj.t^2;
            if loss < 1e-10
                loss = 0;
            end
            info = sprintf( '%sR = %g, T = %g, L = %g', info, obj.rHR^2, obj.t^2, loss );
            if abs(obj.rHR) ~= abs(obj.rAR)
                info = sprintf( '%s, R(AR) = %g', info, obj.rAR^2 );
            end
            info = sprintf( '%s\n', info );
            if obj.invRoC == 0
                info = sprintf( '%sRoC(HR) = flat', info);
            else
                info = sprintf( '%sRoc(HR) = %g', info, 1/obj.invRoC );
            end
            info = sprintf( '%s, Aperture = %f, Thickness = %f, n = %f, Inc.Angle = %g (degree)\n', info, ...
                obj.Aperture, obj.Thickness, obj.nRef, obj.incAngle/pi*180 );
            
            info = sprintf('%swOpt = %g, Wfft = %g\n', ...
                info, obj.wOpt, obj.Wfft );
            
            if obj.HRfiles.numFiles > 0
                info = sprintf( '%s%s\n', info, obj.HRfiles.fileInfo );
            end
            
            if obj.TRfiles.numFiles > 0
                info = sprintf( '%s%s\n', info, obj.TRfiles.fileInfo );
            end
            
            if obj.ARfiles.numFiles > 0
                info = sprintf( '%s%s\n', info, obj.ARfiles.fileInfo );
            end
        end
                
        % misc works
        function setupMisc( obj )
            setupMisc@OpticObj( obj );
            
            obj.updateMirMaps;
        end
        
        %% function updateMirMaps( obj, varargin )
        function updateMirMaps( obj, varargin )
            %% function updateMirMaps( varargin )
            % varargin redefines variables used in the formula provided in dataFileManaher.addMap('fileName', 'formula')
            % varargin is like 'RoC', 100e3, 'delL', 1e-6
            % Variables in varargin override default values provided at the function call
            if ~UtilTK.validArgList( varargin{:} )
                error('variable definition incorrect');
            end
            
            obj.variables = varargin;
            
            % make default reflection and transmission map, which are zeros
            obj.reflFFT = cell(size(obj.inPorts));
            [obj.reflFFT{:}] = deal( obj.ifo.ffts.zeroMat );
            obj.transFFT = cell(size(obj.inPorts));
            [obj.transFFT{:}] = deal( obj.ifo.ffts.zeroMat );
            
            % build maps
            obj.buildMaps;
            
            % aftercare by calling modMap and removing nan in maps
            obj.afterCare;
        end
        
        %% after all maps are built, let modMap to do the final touch, like adding BS baffle
        % nan's are replaced by 0, so reflFFT and transFFT do not have nans
        % reflFFT and transFFT are 0 outside of clear aperture, so outField = mapFFT * inField
        function afterCare( obj )
            if ~isempty( obj.modMap )
                [x, y] = obj.getXYR;
                [obj.reflFFT, obj.transFFT] = obj.modMap( obj.reflFFT, obj.transFFT, x, y, obj.modMapVar );
            end

            % cleanup NaN sorunding the boundary (nan*0 = nan, so aperture mask does not clean)
            for id = 1 : length( obj.reflFFT )
            	obj.reflFFT{id}( isnan( obj.reflFFT{id} ) ) = 0;
               	obj.transFFT{id}( isnan( obj.transFFT{id} ) ) = 0;
            end
        end
        
        %% remote temporal variables for garbage collection
        % remote 4 maps which are not used for field calculation
        function cleanTemp( obj )
            obj.RPYmap = [];
            obj.RSXmap = [];
            obj.TPXmap = [];
            obj.TSYmap = [];
        end
        
        %% rotate optics to make the normally injected field on HR side be reflected without tilt component
        % also curvature mismatch is calculated by using LG10 in the reflected field and minimize curvature mismatch
        % When applyTilt is false, just the tilt correction is calculated, but the mirror is not moved 
        function corrected = doRemoveTiltCurvature( obj, applyTilt )
            corrected = false;

            if ~( obj.removeTilt || obj.removeCurvature )
                return;
            end

            obj.optDCurv = 0;
            obj.optTheta = 0;
            obj.optPhi = 0;
                    
            HRinID = obj.inPorts(1);
            HRoutID = obj.outPorts(1);
            
            % save current fields in the input and output ports, so that they will be restored
            inEFFT = obj.ifo.fields.fieldList(HRinID).EFFT;
            outEFFT = obj.ifo.fields.fieldList(HRoutID).EFFT;
            
            % prepare idealistic field
            obj.ifo.fields.setHGField( HRinID, 0, 0, 1 );
            w = obj.ifo.fields.fieldList(HRoutID).w;
            
            obj.refl( 1, obj.ifo.fields.fieldList(HRinID).EFFT );
            HG00 = obj.ifo.fields.fieldList(HRoutID).HGCoef( 0, 0 );

            if obj.removeTilt
                % direction of outgoing field
                % k w theta cos(phi) = i HG10 = -imag( HG10 )
                % k w theta sin(phi) = i HG01 = -imag( HG01 )
                iHG10 = -imag(obj.ifo.fields.fieldList(HRoutID).HGCoef( 1, 0 )/HG00);
                iHG01 = -imag(obj.ifo.fields.fieldList(HRoutID).HGCoef( 0, 1 )/HG00);

                theta = sqrt( iHG10^2 + iHG01^2 ) / (obj.ifo.fields.kCR*w);
                obj.optTheta = theta;
                obj.optPhi = atan2( iHG01, iHG10 ) + pi;
                corrected = true;
            end
            if obj.removeCurvature
                iLG10 = imag( obj.ifo.fields.fieldList(HRoutID).LGCoef( 1, 0 )/HG00 );
                % LG10 coef = i k w^2/4 (1/Rref - 1/Rmode)
                % 1/Rmir -> 1/Rmir + (1/Rref - 1/Rmode)/2 = 1/Rmir + optDCurv
                obj.optDCurv = iLG10 * 2/( obj.ifo.fields.kCR * w^2 );
                corrected = true;
            end
            
            % restore original fields
            obj.ifo.fields.fieldList(HRinID).EFFT = inEFFT;
            obj.ifo.fields.fieldList(HRoutID).EFFT = outEFFT;
            
            if applyTilt
                [x,y,r] = obj.getXYR;
                Cthick = cos( obj.optTheta ) - 1;
                Cx = cos( obj.optPhi )*sin( obj.optTheta );
                Cy = sin( obj.optPhi )*sin( obj.optTheta );
                delZ = Cthick * obj.Thickness/2 - Cx * x - Cy * y;
                HRCorr = exp( 2i * obj.ifo.fields.kCR * (delZ  + r.^2/2*obj.optDCurv) );
                ARCorr = exp( 2i * obj.ifo.fields.kCR * (delZ  + r.^2/2*obj.optDCurv*obj.nRef) );
                obj.reflFFT{1} = obj.reflFFT{1} .* HRCorr;
                obj.reflFFT{2} = obj.reflFFT{2} .* fliplr( conj(ARCorr) );
                % this is a test to see if the tilt reoval has worked ok
%                obj.doRemoveTiltCurvature( false );
            end
        end
    end
    
    methods (Abstract)
        % abstract function, to be implemented by the derived class
        updateNdep( obj )
    end
    
    methods (Static) 
        % parabalic case : r^2 / 2 RoC
        % spherical case : r^2 / 2 RoC x 2 / ( 1 + sqrt(1 - rsq / RoC^2) )
        function del = delSphPar( rsq, invRoC, spherical )
            del = rsq / 2 * invRoC;
            if spherical
                del = del * 2 ./ ( 1 + sqrt( abs(1 - rsq*invRoC^2) ) );
            end
        end
            
        % distance change on reflection
        % s ~= -(x^2/cos(theta) + y^2*cos(theta))/R
        % cosAlpha ~= cos(incAngle)
        function [s, cosAlpha] = Spherical( invR, x, y, theta )
            g = sqrt( cos(theta)^2 + 2*x*sin(theta)*invR - (x.^2+y.^2)*invR^2);
            f = (2*x*sin(theta) - (x.^2+y.^2)*invR) ./ (cos(theta) + g);
            s = f - (x*sin(2*theta) - f*cos(2*theta)) ./...
                (2*sin(2*theta)*(sin(theta)-x*invR).*g - cos(2*theta)*(1-2*g.^2));
            
            cosAlpha = sqrt( cos(theta).^2 + 2*x* sin(theta)*invR - (x.^2+y.^2)*invR^2 );
        end
        
         % s ~= -(x^2 + y^2)/rho
         % cosAlpha = cos( b/R )
        function [s, cosAlpha] = Parabolic( invR, x, y, theta )
            b = 1/invR * tan(theta);
            a = -2*b*invR;
            beta = 1-(b*invR)^2;
            g = 2./(1 + ((x+b).^2 + y.^2)*invR^2);
            f = -(x.^2 + 2*x*b + y.^2)*invR/2;
            s = f + (a*x - beta*f)./(a*g.*(x+b)*invR + beta*(1-g));
            
            cosAlpha = cos( atan( (x+b)*invR ) );
        end
        
        % reflection by mirror = map * fliplr( inField )
        function outField = reflInOut( reflMap, inField )
            outField = reflMap .* fliplr( inField );
        end
        
        % transmission through mirror = map * inField
        function outField = transInOut( transMap, inField )
            outField = transMap .* inField;
        end

    end

    methods ( Abstract )
        buildMaps( obj )
    end
end
    