% function [data, xl, yl] = loadLMAcol(filename)
%
% Load LMA.col data files with the following data structure
%
%--------------------------
%  656 482	 : number of colomn and lines of the map (Nx, Ny)
%  328 241	 : cordinates of the center of the map, when 0 0, no meaning (x0, y0)
%  0 0       : not used
%  1064.0    : code number equivalent to NaN (if you find this number in
%  the map, means that there is no measurement there)
%  0.00035	 : step of the grid in m
%  === possible string separator
%  : all other lines: values of the map in nm (first line then second line, ...)
%  data11  (x0,    y0) : dat(1,1)
%  data12  (x0+d,  y0) : dat(1,2)
%  data13  (x0+2d, y0) : ...
%  ...
%  data21  (x0,    y0+d) : dat(2,1)
%  data22  (x0+d,  y0+d) : dat(2,2)
%  ...
%--------------------------
%
% This function includes the code to replace the optic data at the center
% in case the center the location is not explicitly given by the 3rd and 4th data in the header,
% This code is copied from Richard Day's 'Rich_Extract_Mirror_Simple_resample.m'
%
% Hiro Yamamoto
% Oct. 19, 2014  Original code from SIS dataLoader and Rich_Extract_Mirror_Simple_resample
%
function [data, xl, yl] = loadLMAcol(filename)

% open the file
[fid, message]=fopen(filename,'r');
if fid == -1
    error('File (''%s'') open failed because %s', filename, message);
end

% read the data
[data,count] = fscanf(fid,'%g ');

if count < 8
    error('data file did not have proper header information');
end

% map(Ny, Nx)
Nx = data(1);
Ny = data(2);

% center of the optic in the map
% If (0,0), they need to be adjusted
x0 = data(3);
y0 = data(4);

% data with this value are not measured, so it means NaN
LMA_NaN = data(7);

% bin size, same for x and y
binSize = data(8);

if count == 8
    % possible seperator line with characters
    fgets(fid);
    [data, count] = fscanf(fid, '%g ');
else
    % remove the header part from the data
    count = count - 8;
    data = data(9:end);
end

% close the file
fclose(fid);

if count < Nx*Ny
    error('The number of data stored (%d) is less than expected (%d)', count, Nx*Ny );
end

% FT_read_virgo_map(filename) by Charlotte Bond, 10.07.2008, says that
% there could be 0 value at the first or end of the data section, which is to be removed
if data(end) == 0
    data = data(1:end-1);
end
if data(1) == 0
    data = data(2:end);
end
count = length( data );

% change the one column data (shown at the top) to 2D
% one column data = dat(1,1), dat(1,2), ..., dat(1,Nx), dat(2,1), dat(2,2) ...
% When reshaped, it is data(Nx,Ny), so transmose to data(Ny,Nx)
% data runs from top to bottom, but physical y axis runs from bottm to top, so flipud
data = flipud(reshape(data, Nx, Ny)');

% fill data with NaN values by NaN
data( data == LMA_NaN ) = NaN;
 
if x0 == 0 || y0 == 0
    % if the center point is not specified, 
    % identify the area which are filled by valid data and
    % use the center of that region as the center
    
    % remove the outer edge
    tmpMask=ones(size(data));
    tmpMask(2:end-1,2:end-1)=0;
    data(tmpMask==1) = NaN;
    % max(dat,[],1/2) = nan for those columns (row) which have only nans
    % remove rows and columns which are only nans
    data(:, isnan(max(data,[],1)) )=[];
    data( isnan(max(data,[],2)) ,:)=[];

    % calculate new size and coordinates with the center (0,0)
    [Ny, Nx] = size(data);

    % don't use the (x0,y0) in the header, but use the center of the data
    x0 = Nx/2 + 0.5;
    y0 = Ny/2 + 0.5;
end

% create the data array
xl = ((1:Nx) - x0) * binSize;
yl = ((1:Ny) - y0) * binSize;

% LMA data is in units of nm
data = data * 1e-9;
