%
% [amap, xl, yl] = loadOneDataFile( fileName, mirrorAperture [0] , ApertureFit [-1], removeZ [2] )
%
% Load data from a file and return data(NY,NX), xl(1,NX) and yl(NY,1)
%
% Datafile format supported are (as of June 2015)
%    Metropro data, Wyko OPD, LMA col and SIS_Nxxx_Wyyy
%
% if mirrorAperture > 0 is provided, data out of this aperture is filled by NaN 
%
% If ApertureFit is given and positive, zernite terms are removed.
%       removeZ = 1 : up to tilt
%               = 2 : up to power <= default
%
% To support a new format data, add the data loader of that format in this function
%
% This function cashes data. To clear the cash, call loadOneDataFile( 'CLEARCASH' )
%
% October 19th, 2014
%   LMA col file loader is changed to loadLMAcol, which correctly handled off-centered maps
%
% October 6th, 2014
%   Default values of ApertureFit and removeZ are explicitly set at the beginning.
%
% August 5th, 2014
%   removeTiltPower is replaced by zernikeCoef
%
% January 10, 2014  HY  
%       1) Wyko OPD format added
%       2) NaNs are removed by average values of surrounding valid data
%

function [amap, xl, yl] = loadOneDataFile( fileName, mirrorAperture, ApertureFit, removeZ )

if nargin == 0 || strcmp( fileName, 'CLEARCASH' )
    clearCash;
    return;
end

if ~exist( 'mirrorAperture', 'var' )
    mirrorAperture = 0;
end

if ~exist( 'ApertureFit', 'var' )
    ApertureFit = -1;
end

if ~exist( 'removeZ', 'var' )
    removeZ = 2;
end

[amap, xl, yl] = findCashed( fileName, mirrorAperture, ApertureFit, removeZ );

if ~isempty( amap )
    return;
end

%% modify this section to add a new data format
if ~isempty( regexpi( fileName, '.col$' ) )
    % LMA col file
    [amap, xl, yl] = loadLMAcol( fileName );
%    [amap, xl, yl] = FT_read_virgo_map_FOGP( fileName );
elseif ~isempty( regexpi( fileName, '.opd$' ) )
    % Wyko OPD file
    [amap, wavelength, aspect, pixel] = readopd( fileName );
    amap( abs(amap) > 1e10 ) = NaN;
    amap = wavelength * amap';
    [ysize, xsize] = size( amap );
    xl = (-xsize/2+0.5:xsize/2-0.5)* pixel;
    yl = (-ysize/2+0.5:ysize/2-0.5)* pixel * aspect;
elseif  ~isempty(regexp( fileName, '_N\d', 'once')) &&  ~isempty(regexp( fileName, '_W\d', 'once'))
    % SIS *_W600_N512 ascii data
    [amap, xl, yl] = loadSISData( fileName );
% elseif ~isempty( regexpi( fileName, '.txt$' ) )
%     % simple text data, either 3 column [x,y,z] or two column [r,z]
%     % for two column case, if r ranges from negative to positive, 
%     % it is assumed to be data along x axis and 2d data is generated by interpolating long phi
%     [amap, xl, yl] = laodTEXTData( fileName );
else
    % for the MetroPro data, if ApertureFit is given, then remove up to curvature
    % this map is to be used with RoCFit, which is the reference used to measure the surface
    [ amap, xl, yl ] = loadMetroProData( fileName );
end

%%
if isempty( amap )
    fprintf( 1, 'File ''%s'' could not be loaded\n', fileName );
else
    if mirrorAperture > 0
        [Xm, Yn] = meshgrid(xl, yl);
        amap( Xm.^2+Yn.^2 > mirrorAperture^2/4 ) = NaN;
    end
    % replace NaNs in the mirror area by averages of surrounding pixels.
    amap = fillNaN( amap );
    
    if ApertureFit > 0
        % aperture is defined in units of the bin size
        % amap = removeTiltPower( amap, ApertureFit, xl(2)-xl(1), yl(2)-yl(1), removeZ );
        % nmList = 2 to remove up to tilt, nmList = 3 to remove up to power
        % just to avoid unknown case
        if removeZ == 1 || removeZ == 2
            amap = zernikeCoef( amap, xl, yl, ApertureFit/2, removeZ + 1 );
        end
    end
    
    addToCash( fileName, mirrorAperture, ApertureFit, removeZ, amap, xl, yl );
end
end

function clearCash
global FOGPRIME13DATA;
FOGPRIME13DATA = [];
end

function [amap, xl, yl] = findCashed( fileName, mirrorAperture, ApertureFit, removeZ )

global FOGPRIME13DATA;

if isempty( FOGPRIME13DATA )
    FOGPRIME13DATA = cell(7,0);
end

amap = [];
xl = [];
yl = [];

for id = 1 : size( FOGPRIME13DATA, 2 )
    if strcmp( FOGPRIME13DATA{1,id}, fileName ) && FOGPRIME13DATA{2,id} == mirrorAperture && ...
            FOGPRIME13DATA{3,id} == ApertureFit && FOGPRIME13DATA{4,id} == removeZ
        amap = FOGPRIME13DATA{5,id};
        xl = FOGPRIME13DATA{6,id};
        yl = FOGPRIME13DATA{7,id};
    end
end

end

function addToCash( fileName, mirrorAperture, ApertureFit, removeZ, amap, xl, yl )

global FOGPRIME13DATA;

id = size( FOGPRIME13DATA, 2 ) + 1;
FOGPRIME13DATA{1,id} = fileName;
FOGPRIME13DATA{2,id} = mirrorAperture;
FOGPRIME13DATA{3,id} = ApertureFit;
FOGPRIME13DATA{4,id} = removeZ;
FOGPRIME13DATA{5,id} = amap;
FOGPRIME13DATA{6,id} = xl;
FOGPRIME13DATA{7,id} = yl;

end