% [dat, coef, nmList] = zernikeCoef( dat, xl, yl, radius, nmList[3], remove[1], normalize[false], w[0] )
%
% Zernike coefficients are calculated using data in radius.
% Those terms can be subtracted from data (remove > 0) or can be summed up (remove < 0).
%
% dat(NY,NX) contains the data and xl(1,NX) or xl(NY,NX) (yl(1,NY) or yl(NY,NX)) is the x (y) axis grid points.
%
% nmList specified which terms are to be calculated.
%   If nmList is not defined, piston, tilt, and power terms are handled
%   nmList(:,1) = n, nmList(:,2) = m
%   nmList = 0 ~ 36 : first nmList+1 Wyko terms are handled (0:piston,1:tiltx,2:tilty,3:power)
%   nmList < 0 : nmList is filled with all indexes whose n is up to abs(nmList), i.e.,
%        (0,0), (1,1), (1,-1), ... (abs(nmList), abs(nmList)), (abs(nmList), abs(nmList)-2), ...(abs(nmList), -abs(nmList))
% on exit, nmList is a list of nominal index pair, i.e., coef(ind) * zernikeFunc( nmList(ind,1), nmList(ind,2), r/r0, phi )
%
% coef(:) is the coefficient corresponding to (n,m) = (nmList(:,1), nmList(:,2))
%
% calculate zernike coefficient for Znm using data in a circle within radius.
% If remove is positive, this zernike term is removed from the map in the ENTIRE area.
%       If remove is negative, the dat returned is the summation of those zernike terms,
%       i.e., dat(out, remove<0) = dat(in) - dat(out,remoe>0)
%
% If normalized is true, zernike function is the one with unit intensity, i.e., integral Znm^2 = 1
% Otherwise, integral Znm^2 = pi/(n+1) * 1  for m=0, *0.5 for m~=0.
%
% if w > 0 is provided, gaussian weight exp(-2*r^2/w^2) is used to calculate the coef
%
% This uses function zernikeFunc.
%
% power term is (n,m) = (2,0), and RoC = radius^2/(4*coeff)
%       for unnormalized zernike(2,0)=2*rho^2-1
%
% December 16, 2014
%   nmList in the form of index pair is returned as output
%
% October 20, 2014
%   Gaussian weight calculation added
%
% July 30, 2014  HY
%   remove < 0 option added. This returns sums of zernike terms.
%   Coefficient calculation takes into account the discritization error by using the intensity in radius calculated numerically
%
%
function [dat, coef, nmList] = zernikeCoef( dat, xl, yl, radius, nmList, remove, normalize, w )
% set defaults
if ~exist( 'remove', 'var' )
    remove = 1;
end
if ~exist( 'normalize', 'var' )
    normalize = false;
end
if ~exist( 'nmList', 'var' )
    % if no nmList, remove up to power
    nmList = 3;
end

% convert nmLsit to a list of zernike index list
nmList = decodeNMList( nmList );

leng = length( nmList );
coef = zeros( leng, 1 );

% prepare 2D xg,yg list
if (min(size(xl)) == 1) && (min(size(yl)) == 1)
    [xg, yg] = meshgrid( xl, yl );
elseif (size(xl,1) == size(yl,1)) && (size(xl,2) == size(yl,2))
    xg = xl;
    yg = yl;
else
    error( 'xl and yl format is not recognized\n');
end

% xg and yg are normalied by the radius
xg = xg / radius; yg = yg / radius;
% ind are indexes of point within the radius
ind0 = find( (xg.^2+yg.^2) <= 1 );
% ind are indexes of point within the radius and not-NaN
ind = find( ~isnan(dat) & ((xg.^2+yg.^2) <= 1) );

if remove < 0
    dat0 = dat;
end

if ~exist('w', 'var') || w < 0
    % unweighted fit or coef(i) = int dxdy dat * zernike(i) / int zernike(i)^2
    for inm = 1 : leng
        % use normalized zernite
        Zfunc = zernikeFunc( nmList(inm,1), nmList(inm,2), ...
            sqrt(xg.^2 + yg.^2), atan2(yg, xg), 1 );
        % this should be 1 analytically. Due to discritization, it may be different, so correct by this
        norm = sum( Zfunc(ind0) .* Zfunc(ind0) );
        % to calculate coefficients, use only points within the radius and not-NaN
        coef(inm) = sum( dat(ind) .* Zfunc(ind) ) / norm;
        
        % if remove, remove all, even outside of radius
        % remove at leach stage so that coef for higher order are calculated numerically accurately, may be
        if remove ~= 0
            dat = dat - Zfunc * coef(inm);
        end
    end
    
else
    % Fit using gaussian shape weight, exp(-2 * r^2 / w^2 )
    % coef are calculated to minimize int dxdy ( dat - sum( coef * zernike ) )^2 * weight
    % M coef = A, M(i,j) = int dxdy zenike(i) zernike(j) weight, A = int dxdy dat * zernike(i) * weight
    Mij = zeros( leng, leng );
    Ai = zeros( leng, 1 );
    
    weight = exp( -2*(xg(ind).^2 + yg(ind).^2)/(w/radius)^2 );
    
    for inmI = 1 : leng
        % use normalized zernite
        ZfuncI = zernikeFunc( nmList(inmI,1), nmList(inmI,2), ...
            sqrt(xg.^2 + yg.^2), atan2(yg, xg), 1 );
        Ai(inmI) = sum( dat(ind) .* ZfuncI(ind) .* weight );
        Mij(inmI,inmI) = sum( ZfuncI(ind).^2 .* weight );
        for inmJ = inmI+1 : leng
            ZfuncJ = zernikeFunc( nmList(inmJ,1), nmList(inmJ,2), ...
                sqrt(xg.^2 + yg.^2), atan2(yg, xg), 1 );
            Mij(inmI,inmJ) = sum( ZfuncI(ind).* ZfuncJ(ind) .* weight );
            Mij(inmJ,inmI) = Mij(inmI,inmJ);
        end
    end
    coef = inv( Mij ) * Ai;
    
    if remove ~= 0
        for inmI = 1 : leng
            % use normalized zernite
            ZfuncI = zernikeFunc( nmList(inmI,1), nmList(inmI,2), ...
                sqrt(xg.^2 + yg.^2), atan2(yg, xg), 1 );
            dat = dat - ZfuncI * coef(inmI);
        end
    end
end

if ~normalize
    % put weight properly for non-normalized case
    for inm = 1 : leng
        scale = pi/(nmList(inm,1)+1);
        if nmList(inm,2) ~= 0
            scale = scale / 2;
        end
        coef(inm) = coef(inm) / sqrt(scale);
    end
end

if remove < 0
    dat = dat0 - dat;
end

return
end

function nmList = decodeNMList( nmList )
% nmList >= 0 specifies Wyko index
% nmList < 0 specifies all Zernike terms up to n = abs(numList)
if isnumeric( nmList )
    if length( nmList ) == 1
        nmList = int16(nmList);
        if nmList >= 0
            % this is interpreted as WykoIndex
            if nmList > 36
                fprintf(1, 'Wyko index max 36 is used\n');
                nmList = 36;
            end
            Wyko = nmList;
            nmList = [ 0,0; 1,1; 1,-1; 2,0; 2,2; 2,-2; 3,1; 3,-1; 4,0; 3,3; 3,-3; ...
                4,2; 4,-2; 5,1; 5,-1; 6,0; 4,4; 4,-4; 5,3; 5,-3; 6,2; 6,-2; ...
                7,1; 7,-1; 8,0; 5,5; 5,-5; 6,4; 6,-4; 7,3; 7,-3; 8,2; 8,-2; ...
                9,1; 9,-1; 10,0; 12,0];
            nmList = nmList(1:Wyko+1,1:2);
        else
            nmax = abs(nmList)+1;
            nmList = zeros((nmax+1)*nmax/2,2);
            ind = 0;
            for in = 1:nmax
                for im = 1:in
                    ind = ind + 1;
                    nmList(ind,1:2) = [in-1, in-2*(im-1)-1];
                end
            end
        end
    end
end
end

