%
% IMCKAGRA : calculate fields in KAGRA IMC
%     input beam => MC1 => MC3 => MC2 => MC1 => MC3
%                              => transmitted
% 
% imc = IMCKAGRA( 'paramName', paramVal, ... )
% any variables about the line
%     eval( UtilTK.setVars( who, true, varargin{:} ) );
% can be used as parameters and can be changed at run time without modiying the code.
% E.g., 
%   imc = IMCKAGRA('Am1',0, 'Am2',0, 'Am3',0)
%       will simulate IMC without surface maps
%   imc = IMCKAGRA('PcoatMC3T', 0.0225)
%       will simulate IMC with all maps and only thermal lens in MC3, no surface thermal bump
%

classdef IMCKAGRA < FFTIFO
    
    methods
        function obj = IMCKAGRA( varargin )
            obj@FFTIFO( varargin{:} );
        end
        
        function defineIFO( obj, varargin )
            %% specification of KAGRA IMC
            
            % specification of aberration
            Am1 = 1;  % MC1 map weight
            Am2 = 1;  % MC2 map weight
            Am3 = 1;  % MC3 map weight
            PcoatMC1 = 0;   % power absorbed by MC1 coating, used for surface aberration
            PcoatMC2 = 0;   % power absorbed by MC2 coating, used for surface aberration
            PcoatMC3 = 0;   % power absorbed by MC3 coating, used for surface aberration
            PcoatMC3T = 0;  % power absorbed by MC1 coating, used for thermal lens in MC3
            
            % IMC optics parameters
            thick = 0.0295;
            Aperture = 0.096;
            RoCMC2 = 37.3;  % used for mode definition
            ARref = 200e-6;
            T_MC1 = 6019e-6;
            T_MC3 = 6098e-6;
            T_MC2 = 4.7e-6;
            L12 = 26.389;
            L13 = 0.5;
            theta2 = atan( L13/2/L12 );
            theta1 = (pi/2 - theta2)/2;
            loss = 0e-6;  % any additional loss of the HR surface

            % change parameters if needed
            eval( UtilTK.setVars( who, true, varargin{:} ) );

            %% add optics
            obj.optics.addLaser( 'src' );
            obj.addMirror( 'MC1', 'invRoC', 0, 'Thick', thick, 'Aperture', Aperture/2, ...
                'T', T_MC1, 'Lhr', loss, 'Lar', loss+ARref, 'incAngle', theta1 );
            obj.addMirror( 'MC3', 'invRoC', 0, 'Thick', thick, 'Aperture', Aperture, ...
                'T', T_MC3, 'Lhr', loss, 'Lar', loss+ARref, 'incAngle', theta1 );
            obj.addMirror( 'MC2', 'invRoC', 1/RoCMC2, 'Thick', 0, 'Aperture', Aperture/2, ...
                'T', T_MC2, 'Lhr', loss, 'incAngle', theta2 );
            
            obj.addDetector( 'refl' );
            obj.addDetector( 'trans' );
            
            %% define connection of optics
            %
            % When addProp('name','A','B') is called, links from A to B and B to A are created
            % So if addProp are used below, two loops, MC1->MC3->MC2->MC1 and MC1->MC2->MC3->MC1,
            % are build. This is necessary when the backscattering like Virgo is to be simulated.
            % Otherwise, there is no harm, but just waste of memory and time just for book keeping.
            obj.addProp( '', 'src', 'MC1-ARx' );
            obj.addOneProp( 'Prop13', 'MC1-HRx', 'MC3-HRy', L13 );
            obj.addOneProp( '', 'MC3-HRx', 'MC2-HRy', L12 );
            obj.addOneProp( '', 'MC2-HRx', 'MC1-HRy', L12 );
            obj.addProp( '', 'MC1-ARy', 'refl' );
            obj.addProp( '', 'MC3-ARy', 'trans' );
            
            %% mirror map details
            
            % map data
            MC2map = './Data/KAGRAMC2_W48mm_N120.dat';
            MC1map = './Data/KAGRA_c14057_1_avge80mm_flat1.col';
            MC3map = './Data/KAGRA_c14057_2_80mm_corrige_plan2.col';
            
            % beam size on each mirror, used for thermal distortion calculation
            [wMC1, wMC2, wMC3] = obj.findBeamSize

            % surface = datamap * weight + thermal disrotion with coating absorption 
            obj.setHRfiles('MC1', MC1map, ...
                'map*Am1 + calcThermal(0, r, 0, PcoatMC1, wMC1, thick, Aperture/2 )', ...
                0, 1e20, 0, 'PcoatMC1', PcoatMC1, 'wMC1', wMC1, 'Am1', Am1, 'thick', thick, 'Aperture', Aperture );
            obj.setHRfiles('MC2', MC2map, ...
                'map*Am2 + calcThermal(0, r, 0, PcoatMC2, wMC2, thick, Aperture/2 )', ...
                0.02, RoCMC2, 0, 'PcoatMC2', PcoatMC2, 'wMC2', wMC2, 'Am2', Am2, 'thick', thick, 'Aperture', Aperture  );
            obj.setHRfiles('MC3', MC3map, ...
                'map*Am3 + calcThermal(0, r, 0, PcoatMC3, wMC3, thick, Aperture/2 )', ...
                0, 1e20, 0, 'PcoatMC3', PcoatMC3, 'wMC3', wMC3, 'Am3', Am3, 'thick', thick, 'Aperture', Aperture  );
            % transmission map
            obj.setTRfiles('MC3', '', ...
                'calcThermal(1, r, 0, PcoatMC3, wMC3, thick, Aperture/2 )', ...
                0, 1e20, 0, 'PcoatMC3T', PcoatMC3T, 'wMC3', wMC3, 'Am3', Am3, 'thick', thick, 'Aperture', Aperture  );
        end
 
        function [wMC1, wMC2, wMC3] = findBeamSize( obj )
            % build cavity and trace modes
            obj.analyzeCavity;
            obj.setupBaseMode;
            % find the beam size. 
            % findNamedField('name',-1) finds the ID of the field, -1 means 'don't do any extra calculation.
            wMC1 = obj.fields.fieldList( obj.fields.findNamedField( 'MC1-HRx-o', -1 ) ).w;
            wMC2 = obj.fields.fieldList( obj.fields.findNamedField( 'MC2-HRx-o', -1 ) ).w;
            wMC3 = obj.fields.fieldList( obj.fields.findNamedField( 'MC3-HRx-o', -1 ) ).w;
        end

        %{
        % if input mode is to be specified, like chaning the input beam waist position ...
        function setupBaseMode( obj, varargin )

            qInv = -7.29384e-06 - 1i*0.0129713;

            eval( UtilTK.setVars( who, true, varargin{:} ) );
            
            root = obj.optics.findPortID( 'src' );
            obj.cavities.fillQvals( root, qInv );
        end
        %}

    end
end