% prm = KAGRAPRM('power', 10) for 10W input, default is 1W

classdef KAGRAPRFPM < FFTIFO
    
    methods
         function obj = KAGRAPRFPM( varargin )
            obj@FFTIFO( varargin{:} );
         end

        function defineIFO( obj, varargin )
            power = 1;
            
            %% bKAGRA parameters
            RoCPRM = 458.1285;  RoCPRM2 = -3.076;   RoCPRM3 = 24.915;
            RoCITMX = 1.9e3;  RoCETMX = 1.9e3;
            RoCITMY = 1.9e3;  RoCETMY = 1.9e3;
            RoCITM = (RoCITMX + RoCITMY)/2;
            RoCETM = (RoCETMX + RoCETMY)/2;
            
            TITM = 0.004;    TETM = 5e-6;
            
            % Without proper test mass maps and HR side scattering loss, 
            % this value is necessary to get the proper PRG
            armLoss = 80e-6;
            
            % d23 was calculated to make the arm mode to match with the curvature of PRM
            % To get this number, uncomment the commented block in setupBaseMode
            % dBSITMX/Y, the distance between BS to ITM HR is corrected to change from 15cm Sapphire to 10cm Sirica
            d23 = -2.3735094e-02;
            dPRC12 = 14.7615 - d23 ; dPRC23 = 11.0661 + d23; dPRC3B = 15.7638;
            dBSITMX = 26.6649 - 0.15*1.754 + 0.1*0.15;  dBSITMY = 23.3351 - 0.15*1.754 + 0.1*0.15;
            thetaPR2 = 0.583/180*pi;  thetaPR3 = 0.686/180*pi;
            
            % This is to calculate the RFSB frequency
            LPRC = dPRC12 + dPRC23 + dPRC3B + (dBSITMX+dBSITMY)/2;
            RF = ConstantTK.c / LPRC;
            fprintf(1, 'LPRC = %20.12g, RF (c/LPRC) = %20.12g\n', LPRC, RF );
            
            ArmLeng = 3000;
            
            ABS = 0.37;
            dBS = 0.00; % this is to use the design optical pathlength, dBS is mostly for loss calculation which is not crucial for now
            ATM = -8;
            BSmapScale = 1;
            
            eval( UtilTK.setVars( who, true, varargin{:} ) );
            
            %% add optics
            obj.addLaser( 'src', power );
            obj.addMirror( 'PRM', 'invRoC', 1/RoCPRM, 'Thick', 0.05, 'T', 0.1 );
            obj.addMirror( 'PRM2', 'invRoC', 1/RoCPRM2, 'T', 500e-6, 'inc', thetaPR2 );
            obj.addMirror( 'PRM3', 'invRoC', 1/RoCPRM3, 'T', 0, 'incAngle', thetaPR3 );
            
            obj.addMirror( 'BS', 'Aperture', ABS, 'Thickness', dBS, ...
                'T', 0.5, 'Lhr', 0, 'Lar', 0, 'incAngle', pi/4 );
            
            % thickness is used to calculate the optical pathlength, which is included in the definition of dBSITMX/Y
            % To make it correct, set dBSITMX/Y the spatial length between BS to ITM AR and this thickness to be the real thickness
            obj.addMirror( 'ITMX', 'invRoC', 1/RoCITM, 'Aperture', ATM, 'Thick', 0, ...
                'T', TITM, 'Lhr', armLoss/2, 'Lar', 0 );
            obj.addMirror( 'ITMY', 'invRoC', 1/RoCITM, 'Aperture', ATM, 'Thick', 0, ...
                'T', TITM, 'Lhr', armLoss/2, 'Lar', 0 );
            
            % ETMX
            obj.addMirror( 'ETMX', 'invRoC', 1/RoCETM, 'Aperture', ATM, 'Thick', 0.1, ...
                'T', TETM, 'Lhr', armLoss/2 );
            % ETMY
            obj.addMirror( 'ETMY', 'invRoC', 1/RoCETM, 'Aperture', ATM, 'Thick', 0.1, ...
                'T', TETM, 'Lhr', armLoss/2 );
            
            obj.addDetector( 'dark' );
            
            %% define test mass structures
            
            obj.setHRfiles( 'BS', './KAGRA/KAGRA-Data/BS/avg_s1-PT_ca350mm.dat', 'map*BSscale', 0.16, 0, 1, 'BSscale', BSmapScale );
            obj.setARfiles( 'BS', './KAGRA/KAGRA-Data/BS/avg_s2-PT_ca350mm.dat', 'map*BSscale', 0.16, 0, -1, 'BSscale', BSmapScale );
            obj.setTRfiles( 'BS', './KAGRA/KAGRA-Data/BS/avg_s1thrus2-PT_ca350mm.dat', 'map*BSscale', 0.16, 0, -1, 'BSscale', BSmapScale );

            % Here, the thickenss of ITM is important to calculate the themal deformation
            % calcThermal(0,...) is surface deformation, calcThermal(1,...) is thermal lens
            % X arm
            % ITM HR surface map
            obj.setHRfiles( 'ITMX', '', ...
                'calcThermal(0, r, PsubIX, PcoatIX, wIX, 0.1, 0.175 ) + r.^2/2*invRoCITMXRH', ...
                0.16, RoCITMX, 0, 'PsubIX', 0', 'PcoatIX', 0, 'wIX', 0.0353, 'invRoCITMXRH', 0 );
            
            % ITM transmission map
            %  setTRfiles defines the transmission map, which is negative is optical path length
            %  To add a converging lens, use r^2/(2*RoC) with RoC > 0
            obj.setTRfiles( 'ITMX', '', ...
                '-calcThermal(1, r, PsubIX, PcoatIX, wIX, 0.1, 0.175 ) + r.^2/2*invRoCITMXRH', ...
                0.16, 0, 0, 'PsubIX', 0', 'PcoatIX', 0, 'wIX', 0.0353, 'invRoCITMXRH', 0 );
            
            % ETM HR surface map
            obj.setHRfiles( 'ETMX', '', ...
                'map + calcThermal(0, r, 0, PcoatEX, wEX, 0.1, 0.175 ) + r.^2/2*invRoCETMXRH', ...
                0.16, RoCETMX, 0, 'PsubIX', 0', 'PcoatEX', 0, 'wEX', 0.0353, 'invRoCETMXRH', 0 );
            
            % Y arm
            obj.setHRfiles( 'ITMY', '', ...
                'calcThermal(0, r, PsubIY, PcoatIY, wIY, 0.1, 0.175 ) + r.^2/2*invRoCITMYRH', ...
                0.16, 0, 0, 'PsubIY', 0', 'PcoatIY', 0, 'wIY', 0.0353, 'invRoCITMYRH', 0 );
            
            obj.setTRfiles( 'ITMY', '', ...
                '-calcThermal(1, r, PsubIY, PcoatIY, wIY, 0.1, 0.175 ) + r.^2/2*invRoCITMYRH', ...
                0.16, 0, 0, 'PsubIY', 0', 'PcoatIY', 0, 'wIY', 0.0353, 'invRoCITMYRH', 0 );
            
            obj.setHRfiles( 'ETMY', '', ...
                'calcThermal(0, r, 0, PcoatEY, wEY, 0.1, 0.175 ) + r.^2/2*invRoCETMYRH', ...
                0.16, RoCETMY, 0, 'PsubIY', 0', 'PcoatEY', 0, 'wEY', 0.0353, 'invRoCETMYRH', 0 );

            %% connect optics
            
            obj.addProp( '', 'src', 'PRM-AR' );
          
            obj.addProp( '', 'PRM-HR', 'PRM2-HRx', dPRC12 );
            obj.addProp( '', 'PRM2-HRy', 'PRM3-HRx', dPRC23 );
            obj.addProp( '', 'PRM3-HRy', 'BS-HRx', dPRC3B );
            
            obj.addProp( '', 'BS-ARx', 'ITMX-AR', dBSITMX);
            obj.addProp( '', 'BS-HRy', 'ITMY-AR', dBSITMY);
            
            obj.addProp( '', 'ITMX-HR', 'ETMX-HR', ArmLeng);
            obj.addProp( '', 'ITMY-HR', 'ETMY-HR', ArmLeng);

            obj.props.addProp( '', 'BS-ARy', 'dark');
        end
        
        function setupBaseMode( obj, varargin )
        %% setupBaseMode( obj, varargin )
        % This should call obj.cavities.fillQvals( rootID, qInv ) to define the mode of a given port
        % fillQvals propagates this mode to all fields in the entire cavity and this mode base is used to calculate mode expansion

        %{ 
            % This block calculates d23 adjustments and the input qInv value
            
            % adjust the PRM2-PRM3 length so that the PRC mode matches with the arm mode
            ITMYHRoutID = obj.optics.findPortID( 'ITMY-HR-o' );
            ITMYHRinID = obj.optics.findPortID( 'ITMY-HR-i' );
            ITMYARoutID = obj.optics.findPortID( 'ITMY-AR-o' );
            ETMYHRoutID = obj.optics.findPortID( 'ETMY-HR-o' );
            BSHRxoutID = obj.optics.findPortID( 'BS-HRx-o' );
            PRM3xoutID = obj.optics.findPortID( 'PRM3-HRx-o' );
            PRM2xoutID = obj.optics.findPortID( 'PRM2-HRx-o'  );
            BSARyoutID = obj.optics.findPortID( 'BS-ARy-o' );
            
            % resonating mode in the arm.  rootArmY is ITMYHRoutID
            [qInv, rootID] = obj.cavities.findCavityMode( [ITMYHRoutID, ETMYHRoutID] );
            sprintf('Mode used to define the modes of the entire IFO : portID = %d, RoC = %g',...
                rootID, 1/real(qInv) )
            
            % find the q value going out of the y arm
            % reverse the q value going into the ITMY HR, then let it goes through ITMY toward AR side
            qInvArmYAR = MMTK.ABCD_qinv( obj.optics.findABCD( ITMYHRinID, ITMYARoutID ), MMTK.reverseDir( qInv ) );
            
            % find the adjustment of the PRM3-PRM2 length
            LadjP = obj.cavities.adjustModeMatch( qInvArmYAR, PRM3xoutID, 0, ...
                [ITMYARoutID, BSHRxoutID, PRM3xoutID, PRM2xoutID ], true );
            
            % find the adjustment of the SRM3-SRM2 length
            %           LadjS = obj.cavities.adjustModeMatch( qInvArmYAR, SRM3xoutID, 0, ...
            %               [ITMYARoutID, BSARyoutID, SRM3xoutID, SRM2xoutID ], true );
            
            %            sprintf('PRM3-PRM2 is adjusted by %g and SRM3-SRM2 by %g', LadjP, LadjS )
            sprintf('PRM3-PRM2 is adjusted by %g', LadjP )
            
        %}
            
            % This is based on the calculation above for the cold case
            laserID = obj.optics.findLaser;
            R = 458.1285/1.44963;
            w = 4.425e-3;
            qInv = -1/R - 1i*1.064e-6/pi/w^2;
            obj.cavities.fillQvals( laserID, qInv );
        end
    end

end