function runFPIFOThermal( inPower )

% many helps defines in FFTIFO object
help FFTIFO
help FFTIFO.addMirror
help FFTIFO.setHRfiles

% build FP object
%% This is cold cavity case with 2.5kW input
%% make the cold one again
fp = FPIFOThermal('power', inPower);

fp.lock
oneInfo( fp, '*** cold FP', 'ETMX-HR-o');

% this is the cold state surface map
ETMHR0 = fp.optics.getHRmap('ETMX', true);
ITMHR0 = fp.optics.getHRmap('ITMX', true);

% loop until stationary state is found
fp = thermalLoop( fp );

% show all necessary information
fp.printAll
% fp.fields.printAll
% fp.optics.printAll

% show field info, field name = 'optic name' - 'HR' or 'AR' - 'i' (for going in) or 'o' (for out going)
oneInfo( fp, '*** hot FP', 'ETMX-HR-o');

% field complex amplitude going to ITM HR. E(iy,ix) is the amplitude at xl(ix,iy)
figure(1)
[E, xl] = fp.getField('ITMX-HR-i');
pcolor( xl, xl, log(abs(E).^2) ); shading flat; axis equal; colorbar;

% ITM and ETM HR surface map
% ETMHR(iy,ix) is the height at x = xl(ix), y = xl(yl). This returns maps without nominal RoC
[ETMHR1, ETMxl] = fp.optics.getHRmap('ETMX', true);
[ITMHR1, ITMxl] = fp.optics.getHRmap('ITMX', true);

% calculate the power term to find the optimal RH correction
% this calculates zernike terms, also removes ...
help zernikeCoef

% use 1.5xw radius central region to find the best curvature
% 3 means to calculate piston, tiltx, tilty and power
[~,cwR] = fp.gaussFit('ETMX-HR-i'); w=(cwR(1).w+cwR(2).w)/2;
[~,coef] = zernikeCoef( ETMHR1, ETMxl, ETMxl, w*1.5, 3);
% this is the effective RoC of ETM surface
ETMRoC = (w*1.5)^2/(4*coef(4));
% do the same for ITM
[~,cwR] = fp.gaussFit('ITMX-HR-i'); w=(cwR(1).w+cwR(2).w)/2;
[~,coef] = zernikeCoef( ITMHR1, ITMxl, ITMxl, w*1.5, 3);
ITMRoC = (w*1.5)^2/(4*coef(4));
% print RoCs
fprintf(1, 'zernike fit of the thermally distorted surface : dRoC(ITM) = %g, dRoC(ETM) = %g\n\n', ITMRoC, ETMRoC);

%% now change the parameters of the ITM and ETM to add the RH correction
% see the formulas in setHRfiles for ITM and ETM
% thermal heating changes the RoC to ITMRoC, so RH subtracts it
fp.updateMirMaps('ITMX', 'invRoCITMXRH', -1/ITMRoC);
fp.updateMirMaps('ETMX', 'invRoCETMXRH', -1/ETMRoC);

% lock and see the mode content in the arm
% loop until stationary state is found
fp = thermalLoop( fp );

oneInfo( fp, '*** hot FP with RH', 'ETMX-HR-o');

% save the surface map
ETMHRRH = fp.optics.getHRmap('ETMX', true);
ITMHRRH = fp.optics.getHRmap('ITMX', true);

%% summary plot
figure(2)
dITMHR = ITMHR1 - ITMHR0;
dITMHR = dITMHR - dITMHR(end/2,end/2);
dETMHR = ETMHR1 - ETMHR0;
dETMHR = dETMHR - dETMHR(end/2,end/2);

plot( ITMxl, ITMHR0(end/2,:)*1e9, 'b', ITMxl, ITMHR1(end/2,:)*1e9, 'bx', ITMxl, ITMHRRH(end/2,:)*1e9, 'b.', ITMxl, dITMHR(end/2,:)*1e9', 'b^', ...
    ETMxl, ETMHR0(end/2,:)*1e9, 'r', ETMxl, ETMHR1(end/2,:)*1e9, 'rx', ETMxl, ETMHRRH(end/2,:)*1e9, 'r.', ETMxl, dETMHR(end/2,:)*1e9', 'r^')
axis([-0.16 0.16 -40 10])
legend('ITM cold', 'ITM hot', 'ITM hot+RH', 'ITM hot-cold', 'ETM cold', 'ETM hot', 'ETM hot+RH', 'ETM hot-cold', 'location', 'south');

function oneInfo( fp, message, fieldName )
fprintf(1,'\n\n%s : Inforamtion about field %s\n', message, fieldName);
% power
P = fp.power( fieldName );
% LG mode fraction
LGfrac = abs(fp.LGCoef(fieldName, 0:5,0)).^2 / P;
% round trip loss
RTL = fp.roundtripLoss( 'ETMX-HR-o' );
% curvature of the reflected CR field (prompt reflection + leak from the cavity)
[~,cwR] = fp.gaussFit('ITMX-AR-o'); RoCCR = 2*pi/1.064e-6*(1/cwR(1).invRk + 1/cwR(2).invRk);
% curvature of the reflected SB field, i..e, prompt reflection only
SBRefID = fp.refl( 'ITMX-AR-i' );
[~,cwR] = fp.gaussFit(SBRefID); RoCSB = 2*pi/1.064e-6*(1/cwR(1).invRk + 1/cwR(2).invRk);

fprintf(1,'power = %g, HOM = %g, RTL = %g, RoC(CR) = %g, RoC(SB) = %g\nLG(1:5,0) fraction %g, %g, %g, %g, %g\n\n', P, 1-LGfrac(1), RTL, RoCCR, RoCSB, LGfrac(2:6) );

